
module MenhirBasics = struct
  
  exception Error = Parsing.Parse_error
  
  let _eRR =
    fun _s ->
      raise Error
  
  type token = 
    | WITH
    | VALUE of (
# 73 "lib/stateParser.mly"
       (string)
# 16 "lib/stateParser.ml"
  )
    | TRUE
    | TOK_TAG
    | TOK_PTE
    | TOK_PA
    | TOK_OA
    | TOK_NOP
    | TOKAND
    | SYMB_REG of (
# 69 "lib/stateParser.mly"
       (string)
# 28 "lib/stateParser.ml"
  )
    | STAR
    | SEMI
    | RPAR
    | RCURLY
    | RBRK
    | PTX_REG_TYPE of (
# 88 "lib/stateParser.mly"
       (string)
# 38 "lib/stateParser.ml"
  )
    | PTX_REG_DEC
    | PROC of (
# 68 "lib/stateParser.mly"
       (int)
# 44 "lib/stateParser.ml"
  )
    | PLUS
    | OR
    | OBSERVED
    | NUM of (
# 72 "lib/stateParser.mly"
       (string)
# 52 "lib/stateParser.ml"
  )
    | NOTEQUAL
    | NOT
    | NAME of (
# 70 "lib/stateParser.mly"
       (string)
# 59 "lib/stateParser.ml"
  )
    | LPAR
    | LOCATIONS
    | LCURLY
    | LBRK
    | INSTR of (
# 86 "lib/stateParser.mly"
       (string)
# 68 "lib/stateParser.ml"
  )
    | IMPLIES
    | FORALL
    | FINAL
    | FILTER
    | FAULT
    | FALSE
    | EXISTS
    | EQUALEQUAL
    | EQUAL
    | EOF
    | DOLLARNAME of (
# 71 "lib/stateParser.mly"
       (string)
# 83 "lib/stateParser.ml"
  )
    | COMMA
    | COLON
    | ATTRS
    | ATOMICINIT
    | ATOMIC
    | AND
    | AMPER
  
end

include MenhirBasics

# 1 "lib/stateParser.mly"
  
(****************************************************************************)
(*                           the diy toolsuite                              *)
(*                                                                          *)
(* Jade Alglave, University College London, UK.                             *)
(* Luc Maranget, INRIA Paris-Rocquencourt, France.                          *)
(*                                                                          *)
(* Copyright 2010-present Institut National de Recherche en Informatique et *)
(* en Automatique, ARM Ltd and the authors. All rights reserved.            *)
(*                                                                          *)
(* This software is governed by the CeCILL-B license under French law and   *)
(* abiding by the rules of distribution of free software. You can use,      *)
(* modify and/ or redistribute the software under the terms of the CeCILL-B *)
(* license as circulated by CEA, CNRS and INRIA at the following URL        *)
(* "http://www.cecill.info". We also give a copy in LICENSE.txt.            *)
(****************************************************************************)

open Constant
open TestType
open LocationsItem
open MiscParser
open ConstrGen

let mk_sym_tag s t =
  Symbolic (Virtual {default_symbolic_data with name=s;tag=Some t;})

let do_mk_sym_tagloc s o =
  if
    o < 0 ||
    o mod MachSize.granule_nbytes <> 0
  then raise Parsing.Parse_error ;
  Symbolic (TagAddr (VIR,s,o))

let mk_sym_tagloc s o =
  let o =
    try int_of_string o
    with Invalid_argument _ -> raise Parsing.Parse_error in
  do_mk_sym_tagloc s o

let mk_sym_tagloc_zero s = do_mk_sym_tagloc s 0

let mk_sym_morello p s t =
  let p_int = Misc.string_as_int64 p in
  if
    not (Int64.equal (Int64.logand p_int 0x7L) 0L)
    || Int64.compare p_int (Int64.shift_left 1L 36) >= 0
    || Int64.compare p_int 0L < 0
    then Printf.eprintf "Warning: incorrect address encoding: %#Lx\n" p_int ;
  let truncated_perms = Int64.shift_right_logical p_int 3 in
  let tag = if Misc.string_as_int t <> 0 then 1L else 0L in
  Symbolic
    (Virtual
       {default_symbolic_data
       with
       name=s;
       cap=Int64.logor truncated_perms (Int64.shift_left tag 33); })

let mk_sym_with_index s i =
  Symbolic
    (Virtual
       {default_symbolic_data
       with name=s; offset=Misc.string_as_int i})

let mk_lab p s = Label (p,s)

# 163 "lib/stateParser.ml"

type ('s, 'r) _menhir_state = 
  | MenhirState000 : ('s, _menhir_box_constraints) _menhir_state
    (** State 000.
        Stack shape : .
        Start symbol: constraints. *)

  | MenhirState002 : (('s, 'r) _menhir_cell1_LOCATIONS, 'r) _menhir_state
    (** State 002.
        Stack shape : LOCATIONS.
        Start symbol: <undetermined>. *)

  | MenhirState026 : (('s, 'r) _menhir_cell1_PROC, 'r) _menhir_state
    (** State 026.
        Stack shape : PROC.
        Start symbol: <undetermined>. *)

  | MenhirState029 : ((('s, 'r) _menhir_cell1_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 029.
        Stack shape : PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState035 : (('s, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 035.
        Stack shape : NUM.
        Start symbol: <undetermined>. *)

  | MenhirState038 : ((('s, 'r) _menhir_cell1_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 038.
        Stack shape : NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState044 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 044.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState045 : ((('s, 'r) _menhir_cell1_NAME, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 045.
        Stack shape : NAME LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState065 : (('s, 'r) _menhir_cell1_FAULT _menhir_cell0_lbl _menhir_cell0_NAME, 'r) _menhir_state
    (** State 065.
        Stack shape : FAULT lbl NAME.
        Start symbol: <undetermined>. *)

  | MenhirState067 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 067.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState073 : (('s, 'r) _menhir_cell1_FAULT _menhir_cell0_lbl _menhir_cell0_fault_loc, 'r) _menhir_state
    (** State 073.
        Stack shape : FAULT lbl fault_loc.
        Start symbol: <undetermined>. *)

  | MenhirState089 : (('s, 'r) _menhir_cell1_loc_item, 'r) _menhir_state
    (** State 089.
        Stack shape : loc_item.
        Start symbol: <undetermined>. *)

  | MenhirState093 : (('s, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_state
    (** State 093.
        Stack shape : locations.
        Start symbol: constraints. *)

  | MenhirState094 : (('s, 'r) _menhir_cell1_FILTER, 'r) _menhir_state
    (** State 094.
        Stack shape : FILTER.
        Start symbol: <undetermined>. *)

  | MenhirState096 : (('s, 'r) _menhir_cell1_NOT, 'r) _menhir_state
    (** State 096.
        Stack shape : NOT.
        Start symbol: <undetermined>. *)

  | MenhirState097 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
    (** State 097.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

  | MenhirState098 : (('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 098.
        Stack shape : LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState100 : ((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 100.
        Stack shape : LBRK NUM.
        Start symbol: <undetermined>. *)

  | MenhirState102 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 102.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState105 : ((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_state
    (** State 105.
        Stack shape : LBRK location_global.
        Start symbol: <undetermined>. *)

  | MenhirState108 : (('s, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 108.
        Stack shape : NUM.
        Start symbol: <undetermined>. *)

  | MenhirState110 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 110.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState119 : ((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_state
    (** State 119.
        Stack shape : LBRK location_global.
        Start symbol: <undetermined>. *)

  | MenhirState122 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_PROC, 'r) _menhir_state
    (** State 122.
        Stack shape : LBRK location_global PROC.
        Start symbol: <undetermined>. *)

  | MenhirState125 : ((((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 125.
        Stack shape : LBRK location_global PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState129 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 129.
        Stack shape : LBRK location_global NUM.
        Start symbol: <undetermined>. *)

  | MenhirState132 : ((((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 132.
        Stack shape : LBRK location_global NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState135 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
    (** State 135.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

  | MenhirState147 : (('s, 'r) _menhir_cell1_TOK_OA _menhir_cell0_output_address, 'r) _menhir_state
    (** State 147.
        Stack shape : TOK_OA output_address.
        Start symbol: <undetermined>. *)

  | MenhirState148 : (('s, 'r) _menhir_cell1_COMMA, 'r) _menhir_state
    (** State 148.
        Stack shape : COMMA.
        Start symbol: <undetermined>. *)

  | MenhirState149 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 149.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState153 : ((('s, 'r) _menhir_cell1_NAME, 'r) _menhir_cell1_COLON _menhir_cell0_name_or_num, 'r) _menhir_state
    (** State 153.
        Stack shape : NAME COLON name_or_num.
        Start symbol: <undetermined>. *)

  | MenhirState158 : (('s, 'r) _menhir_cell1_ATTRS, 'r) _menhir_state
    (** State 158.
        Stack shape : ATTRS.
        Start symbol: <undetermined>. *)

  | MenhirState160 : (('s, 'r) _menhir_cell1_NAME, 'r) _menhir_state
    (** State 160.
        Stack shape : NAME.
        Start symbol: <undetermined>. *)

  | MenhirState163 : ((('s, 'r) _menhir_cell1_ATTRS, 'r) _menhir_cell1_separated_nonempty_list_COMMA_NAME_, 'r) _menhir_state
    (** State 163.
        Stack shape : ATTRS separated_nonempty_list(COMMA,NAME).
        Start symbol: <undetermined>. *)

  | MenhirState169 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 169.
        Stack shape : LBRK location_global LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState174 : ((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_state
    (** State 174.
        Stack shape : LBRK location_global.
        Start symbol: <undetermined>. *)

  | MenhirState177 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_PROC, 'r) _menhir_state
    (** State 177.
        Stack shape : LBRK location_global PROC.
        Start symbol: <undetermined>. *)

  | MenhirState180 : ((((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 180.
        Stack shape : LBRK location_global PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState184 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 184.
        Stack shape : LBRK location_global NUM.
        Start symbol: <undetermined>. *)

  | MenhirState187 : ((((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 187.
        Stack shape : LBRK location_global NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState190 : (((('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_cell1_location_global, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 190.
        Stack shape : LBRK location_global LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState198 : (('s, 'r) _menhir_cell1_prop, 'r) _menhir_state
    (** State 198.
        Stack shape : prop.
        Start symbol: <undetermined>. *)

  | MenhirState200 : (('s, 'r) _menhir_cell1_prop, 'r) _menhir_state
    (** State 200.
        Stack shape : prop.
        Start symbol: <undetermined>. *)

  | MenhirState203 : (('s, 'r) _menhir_cell1_locindex, 'r) _menhir_state
    (** State 203.
        Stack shape : locindex.
        Start symbol: <undetermined>. *)

  | MenhirState205 : (('s, 'r) _menhir_cell1_locindex, 'r) _menhir_state
    (** State 205.
        Stack shape : locindex.
        Start symbol: <undetermined>. *)

  | MenhirState207 : (('s, 'r) _menhir_cell1_locindex, 'r) _menhir_state
    (** State 207.
        Stack shape : locindex.
        Start symbol: <undetermined>. *)

  | MenhirState210 : (('s, 'r) _menhir_cell1_location, 'r) _menhir_state
    (** State 210.
        Stack shape : location.
        Start symbol: <undetermined>. *)

  | MenhirState211 : ((('s, 'r) _menhir_cell1_location, 'r) _menhir_cell1_LCURLY, 'r) _menhir_state
    (** State 211.
        Stack shape : location LCURLY.
        Start symbol: <undetermined>. *)

  | MenhirState213 : (('s, 'r) _menhir_cell1_maybev_notag, 'r) _menhir_state
    (** State 213.
        Stack shape : maybev_notag.
        Start symbol: <undetermined>. *)

  | MenhirState218 : (('s, 'r) _menhir_cell1_location, 'r) _menhir_state
    (** State 218.
        Stack shape : location.
        Start symbol: <undetermined>. *)

  | MenhirState223 : (('s, 'r) _menhir_cell1_STAR _menhir_cell0_PROC, 'r) _menhir_state
    (** State 223.
        Stack shape : STAR PROC.
        Start symbol: <undetermined>. *)

  | MenhirState226 : ((('s, 'r) _menhir_cell1_STAR _menhir_cell0_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 226.
        Stack shape : STAR PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState230 : (('s, 'r) _menhir_cell1_STAR _menhir_cell0_NUM, 'r) _menhir_state
    (** State 230.
        Stack shape : STAR NUM.
        Start symbol: <undetermined>. *)

  | MenhirState233 : ((('s, 'r) _menhir_cell1_STAR _menhir_cell0_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 233.
        Stack shape : STAR NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState238 : (('s, 'r) _menhir_cell1_PROC, 'r) _menhir_state
    (** State 238.
        Stack shape : PROC.
        Start symbol: <undetermined>. *)

  | MenhirState241 : ((('s, 'r) _menhir_cell1_PROC, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 241.
        Stack shape : PROC PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState245 : (('s, 'r) _menhir_cell1_NUM, 'r) _menhir_state
    (** State 245.
        Stack shape : NUM.
        Start symbol: <undetermined>. *)

  | MenhirState248 : ((('s, 'r) _menhir_cell1_NUM, 'r) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE, 'r) _menhir_state
    (** State 248.
        Stack shape : NUM PTX_REG_DEC PTX_REG_TYPE.
        Start symbol: <undetermined>. *)

  | MenhirState251 : ((('s, 'r) _menhir_cell1_location, 'r) _menhir_cell1_LCURLY, 'r) _menhir_state
    (** State 251.
        Stack shape : location LCURLY.
        Start symbol: <undetermined>. *)

  | MenhirState254 : (('s, 'r) _menhir_cell1_LBRK, 'r) _menhir_state
    (** State 254.
        Stack shape : LBRK.
        Start symbol: <undetermined>. *)

  | MenhirState260 : (('s, 'r) _menhir_cell1_location, 'r) _menhir_state
    (** State 260.
        Stack shape : location.
        Start symbol: <undetermined>. *)

  | MenhirState262 : ((('s, 'r) _menhir_cell1_location, 'r) _menhir_cell1_LCURLY, 'r) _menhir_state
    (** State 262.
        Stack shape : location LCURLY.
        Start symbol: <undetermined>. *)

  | MenhirState272 : (('s, 'r) _menhir_cell1_prop, 'r) _menhir_state
    (** State 272.
        Stack shape : prop.
        Start symbol: <undetermined>. *)

  | MenhirState276 : ((('s, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter, _menhir_box_constraints) _menhir_state
    (** State 276.
        Stack shape : locations filter.
        Start symbol: constraints. *)

  | MenhirState277 : (('s, 'r) _menhir_cell1_OBSERVED, 'r) _menhir_state
    (** State 277.
        Stack shape : OBSERVED.
        Start symbol: <undetermined>. *)

  | MenhirState279 : (('s, 'r) _menhir_cell1_obsone, 'r) _menhir_state
    (** State 279.
        Stack shape : obsone.
        Start symbol: <undetermined>. *)

  | MenhirState282 : (('s, 'r) _menhir_cell1_atom_prop, 'r) _menhir_state
    (** State 282.
        Stack shape : atom_prop.
        Start symbol: <undetermined>. *)

  | MenhirState286 : (('s, 'r) _menhir_cell1_NOT, 'r) _menhir_state
    (** State 286.
        Stack shape : NOT.
        Start symbol: <undetermined>. *)

  | MenhirState288 : (('s, 'r) _menhir_cell1_LPAR, 'r) _menhir_state
    (** State 288.
        Stack shape : LPAR.
        Start symbol: <undetermined>. *)

  | MenhirState291 : (('s, 'r) _menhir_cell1_FORALL, 'r) _menhir_state
    (** State 291.
        Stack shape : FORALL.
        Start symbol: <undetermined>. *)

  | MenhirState293 : (('s, 'r) _menhir_cell1_FINAL, 'r) _menhir_state
    (** State 293.
        Stack shape : FINAL.
        Start symbol: <undetermined>. *)

  | MenhirState295 : (('s, 'r) _menhir_cell1_EXISTS, 'r) _menhir_state
    (** State 295.
        Stack shape : EXISTS.
        Start symbol: <undetermined>. *)

  | MenhirState300 : (((('s, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter, _menhir_box_constraints) _menhir_cell1_final, _menhir_box_constraints) _menhir_state
    (** State 300.
        Stack shape : locations filter final.
        Start symbol: constraints. *)

  | MenhirState309 : (('s, _menhir_box_constraints) _menhir_cell1_kind, _menhir_box_constraints) _menhir_state
    (** State 309.
        Stack shape : kind.
        Start symbol: constraints. *)

  | MenhirState314 : ('s, _menhir_box_init) _menhir_state
    (** State 314.
        Stack shape : .
        Start symbol: init. *)

  | MenhirState315 : (('s, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_state
    (** State 315.
        Stack shape : STAR.
        Start symbol: init. *)

  | MenhirState316 : (('s, _menhir_box_init) _menhir_cell1_LBRK, _menhir_box_init) _menhir_state
    (** State 316.
        Stack shape : LBRK.
        Start symbol: init. *)

  | MenhirState321 : ((('s, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 321.
        Stack shape : STAR left_loc.
        Start symbol: init. *)

  | MenhirState323 : (((('s, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_amperopt, _menhir_box_init) _menhir_state
    (** State 323.
        Stack shape : STAR left_loc amperopt.
        Start symbol: init. *)

  | MenhirState326 : (('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_state
    (** State 326.
        Stack shape : NAME.
        Start symbol: init. *)

  | MenhirState327 : ((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_state
    (** State 327.
        Stack shape : NAME STAR.
        Start symbol: init. *)

  | MenhirState329 : (((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 329.
        Stack shape : NAME STAR left_loc.
        Start symbol: init. *)

  | MenhirState330 : ((((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_amperopt, _menhir_box_init) _menhir_state
    (** State 330.
        Stack shape : NAME STAR left_loc amperopt.
        Start symbol: init. *)

  | MenhirState332 : ((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_LBRK, _menhir_box_init) _menhir_state
    (** State 332.
        Stack shape : NAME LBRK.
        Start symbol: init. *)

  | MenhirState335 : ((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 335.
        Stack shape : NAME left_loc.
        Start symbol: init. *)

  | MenhirState337 : (((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_ATOMICINIT, _menhir_box_init) _menhir_state
    (** State 337.
        Stack shape : NAME left_loc ATOMICINIT.
        Start symbol: init. *)

  | MenhirState345 : ((('s, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_arrayspec, _menhir_box_init) _menhir_state
    (** State 345.
        Stack shape : NAME arrayspec.
        Start symbol: init. *)

  | MenhirState349 : (('s, _menhir_box_init) _menhir_cell1_ATOMIC _menhir_cell0_NAME, _menhir_box_init) _menhir_state
    (** State 349.
        Stack shape : ATOMIC NAME.
        Start symbol: init. *)

  | MenhirState351 : ((('s, _menhir_box_init) _menhir_cell1_ATOMIC _menhir_cell0_NAME, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 351.
        Stack shape : ATOMIC NAME left_loc.
        Start symbol: init. *)

  | MenhirState354 : (('s, _menhir_box_init) _menhir_cell1_locindex, _menhir_box_init) _menhir_state
    (** State 354.
        Stack shape : locindex.
        Start symbol: init. *)

  | MenhirState358 : (('s, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_state
    (** State 358.
        Stack shape : left_loc.
        Start symbol: init. *)

  | MenhirState363 : ((('s, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_NUM, _menhir_box_init) _menhir_state
    (** State 363.
        Stack shape : left_loc NUM.
        Start symbol: init. *)

  | MenhirState373 : (('s, _menhir_box_init) _menhir_cell1_atom_init, _menhir_box_init) _menhir_state
    (** State 373.
        Stack shape : atom_init.
        Start symbol: init. *)

  | MenhirState376 : ('s, _menhir_box_main_constr) _menhir_state
    (** State 376.
        Stack shape : .
        Start symbol: main_constr. *)

  | MenhirState380 : ('s, _menhir_box_main_filter) _menhir_state
    (** State 380.
        Stack shape : .
        Start symbol: main_filter. *)

  | MenhirState384 : ('s, _menhir_box_main_loc_constr) _menhir_state
    (** State 384.
        Stack shape : .
        Start symbol: main_loc_constr. *)

  | MenhirState386 : (('s, _menhir_box_main_loc_constr) _menhir_cell1_locations, _menhir_box_main_loc_constr) _menhir_state
    (** State 386.
        Stack shape : locations.
        Start symbol: main_loc_constr. *)

  | MenhirState390 : ('s, _menhir_box_main_location) _menhir_state
    (** State 390.
        Stack shape : .
        Start symbol: main_location. *)

  | MenhirState394 : ('s, _menhir_box_pteval) _menhir_state
    (** State 394.
        Stack shape : .
        Start symbol: pteval. *)

  | MenhirState395 : (('s, _menhir_box_pteval) _menhir_cell1_LPAR, _menhir_box_pteval) _menhir_state
    (** State 395.
        Stack shape : LPAR.
        Start symbol: pteval. *)

  | MenhirState399 : ('s, _menhir_box_skip_loc_constr) _menhir_state
    (** State 399.
        Stack shape : .
        Start symbol: skip_loc_constr. *)

  | MenhirState401 : (('s, _menhir_box_skip_loc_constr) _menhir_cell1_locations, _menhir_box_skip_loc_constr) _menhir_state
    (** State 401.
        Stack shape : locations.
        Start symbol: skip_loc_constr. *)


and ('s, 'r) _menhir_cell1_amperopt = 
  | MenhirCell1_amperopt of 's * ('s, 'r) _menhir_state * (unit)

and ('s, 'r) _menhir_cell1_arrayspec = 
  | MenhirCell1_arrayspec of 's * ('s, 'r) _menhir_state * (MiscParser.location * int)

and ('s, 'r) _menhir_cell1_atom_init = 
  | MenhirCell1_atom_init of 's * ('s, 'r) _menhir_state * (MiscParser.state_atom)

and ('s, 'r) _menhir_cell1_atom_prop = 
  | MenhirCell1_atom_prop of 's * ('s, 'r) _menhir_state * (MiscParser.prop)

and 's _menhir_cell0_fault_loc = 
  | MenhirCell0_fault_loc of 's * (ParsedConstant.v)

and ('s, 'r) _menhir_cell1_filter = 
  | MenhirCell1_filter of 's * ('s, 'r) _menhir_state * (MiscParser.prop option)

and ('s, 'r) _menhir_cell1_final = 
  | MenhirCell1_final of 's * ('s, 'r) _menhir_state * (MiscParser.constr)

and ('s, 'r) _menhir_cell1_kind = 
  | MenhirCell1_kind of 's * ('s, 'r) _menhir_state * (string * MiscParser.quantifier)

and 's _menhir_cell0_lbl = 
  | MenhirCell0_lbl of 's * (int * string option)

and ('s, 'r) _menhir_cell1_left_loc = 
  | MenhirCell1_left_loc of 's * ('s, 'r) _menhir_state * (MiscParser.location)

and ('s, 'r) _menhir_cell1_loc_item = 
  | MenhirCell1_loc_item of 's * ('s, 'r) _menhir_state * ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t)

and ('s, 'r) _menhir_cell1_location = 
  | MenhirCell1_location of 's * ('s, 'r) _menhir_state * (MiscParser.location)

and ('s, 'r) _menhir_cell1_location_global = 
  | MenhirCell1_location_global of 's * ('s, 'r) _menhir_state * (ParsedConstant.v)

and ('s, 'r) _menhir_cell1_locations = 
  | MenhirCell1_locations of 's * ('s, 'r) _menhir_state * ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list)

and ('s, 'r) _menhir_cell1_locindex = 
  | MenhirCell1_locindex of 's * ('s, 'r) _menhir_state * (MiscParser.location ConstrGen.rloc)

and ('s, 'r) _menhir_cell1_maybev_notag = 
  | MenhirCell1_maybev_notag of 's * ('s, 'r) _menhir_state * (ParsedConstant.v)

and 's _menhir_cell0_name_or_num = 
  | MenhirCell0_name_or_num of 's * (string)

and ('s, 'r) _menhir_cell1_obsone = 
  | MenhirCell1_obsone of 's * ('s, 'r) _menhir_state * (MiscParser.prop list)

and 's _menhir_cell0_output_address = 
  | MenhirCell0_output_address of 's * (OutputAddress.t)

and ('s, 'r) _menhir_cell1_prop = 
  | MenhirCell1_prop of 's * ('s, 'r) _menhir_state * (MiscParser.prop)

and ('s, 'r) _menhir_cell1_separated_nonempty_list_COMMA_NAME_ = 
  | MenhirCell1_separated_nonempty_list_COMMA_NAME_ of 's * ('s, 'r) _menhir_state * (string list)

and ('s, 'r) _menhir_cell1_ATOMIC = 
  | MenhirCell1_ATOMIC of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_ATOMICINIT = 
  | MenhirCell1_ATOMICINIT of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_ATTRS = 
  | MenhirCell1_ATTRS of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_COLON = 
  | MenhirCell1_COLON of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_COMMA = 
  | MenhirCell1_COMMA of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_EXISTS = 
  | MenhirCell1_EXISTS of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_FAULT = 
  | MenhirCell1_FAULT of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_FILTER = 
  | MenhirCell1_FILTER of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_FINAL = 
  | MenhirCell1_FINAL of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_FORALL = 
  | MenhirCell1_FORALL of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_LBRK = 
  | MenhirCell1_LBRK of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_LCURLY = 
  | MenhirCell1_LCURLY of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_LOCATIONS = 
  | MenhirCell1_LOCATIONS of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_LPAR = 
  | MenhirCell1_LPAR of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_NAME = 
  | MenhirCell1_NAME of 's * ('s, 'r) _menhir_state * (
# 70 "lib/stateParser.mly"
       (string)
# 791 "lib/stateParser.ml"
)

and 's _menhir_cell0_NAME = 
  | MenhirCell0_NAME of 's * (
# 70 "lib/stateParser.mly"
       (string)
# 798 "lib/stateParser.ml"
)

and ('s, 'r) _menhir_cell1_NOT = 
  | MenhirCell1_NOT of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_NUM = 
  | MenhirCell1_NUM of 's * ('s, 'r) _menhir_state * (
# 72 "lib/stateParser.mly"
       (string)
# 808 "lib/stateParser.ml"
)

and 's _menhir_cell0_NUM = 
  | MenhirCell0_NUM of 's * (
# 72 "lib/stateParser.mly"
       (string)
# 815 "lib/stateParser.ml"
)

and ('s, 'r) _menhir_cell1_OBSERVED = 
  | MenhirCell1_OBSERVED of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_PROC = 
  | MenhirCell1_PROC of 's * ('s, 'r) _menhir_state * (
# 68 "lib/stateParser.mly"
       (int)
# 825 "lib/stateParser.ml"
)

and 's _menhir_cell0_PROC = 
  | MenhirCell0_PROC of 's * (
# 68 "lib/stateParser.mly"
       (int)
# 832 "lib/stateParser.ml"
)

and ('s, 'r) _menhir_cell1_PTX_REG_DEC = 
  | MenhirCell1_PTX_REG_DEC of 's * ('s, 'r) _menhir_state

and 's _menhir_cell0_PTX_REG_TYPE = 
  | MenhirCell0_PTX_REG_TYPE of 's * (
# 88 "lib/stateParser.mly"
       (string)
# 842 "lib/stateParser.ml"
)

and ('s, 'r) _menhir_cell1_STAR = 
  | MenhirCell1_STAR of 's * ('s, 'r) _menhir_state

and ('s, 'r) _menhir_cell1_TOK_OA = 
  | MenhirCell1_TOK_OA of 's * ('s, 'r) _menhir_state

and _menhir_box_skip_loc_constr = 
  | MenhirBox_skip_loc_constr of (MiscParser.constr) [@@unboxed]

and _menhir_box_pteval = 
  | MenhirBox_pteval of (ParsedPteVal.t) [@@unboxed]

and _menhir_box_main_location = 
  | MenhirBox_main_location of (MiscParser.location) [@@unboxed]

and _menhir_box_main_loc_constr = 
  | MenhirBox_main_loc_constr of ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.constr) [@@unboxed]

and _menhir_box_main_filter = 
  | MenhirBox_main_filter of (MiscParser.prop option) [@@unboxed]

and _menhir_box_main_constr = 
  | MenhirBox_main_constr of (MiscParser.constr) [@@unboxed]

and _menhir_box_init = 
  | MenhirBox_init of (MiscParser.state) [@@unboxed]

and _menhir_box_constraints = 
  | MenhirBox_constraints of ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.prop option * MiscParser.constr *
  (string * MiscParser.quantifier) list) [@@unboxed]

let _menhir_action_008 =
  fun () ->
    (
# 259 "lib/stateParser.mly"
        ( () )
# 883 "lib/stateParser.ml"
     : (unit))

let _menhir_action_009 =
  fun () ->
    (
# 260 "lib/stateParser.mly"
  ( () )
# 891 "lib/stateParser.ml"
     : (unit))

let _menhir_action_010 =
  fun _1 _3 ->
    (
# 397 "lib/stateParser.mly"
    ( (Location_global (Constant.mk_sym _1),Misc.string_as_int _3) )
# 899 "lib/stateParser.ml"
     : (MiscParser.location * int))

let _menhir_action_011 =
  fun _1 ->
    (
# 218 "lib/stateParser.mly"
           ((_1,ParsedConstant.zero))
# 907 "lib/stateParser.ml"
     : (MiscParser.location * ParsedConstant.v))

let _menhir_action_012 =
  fun _1 _3 ->
    (
# 219 "lib/stateParser.mly"
                              ((_1,_3))
# 915 "lib/stateParser.ml"
     : (MiscParser.location * ParsedConstant.v))

let _menhir_action_013 =
  fun _1 ->
    (
# 225 "lib/stateParser.mly"
       ( let x,v = _1 in x,(TyDef,v) )
# 923 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_014 =
  fun loc typ ->
    (
# 226 "lib/stateParser.mly"
                         ( (loc, (Ty typ,ParsedConstant.zero)) )
# 931 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_015 =
  fun loc typ ->
    (
# 227 "lib/stateParser.mly"
                               ( loc,(Atomic typ,ParsedConstant.zero))
# 939 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_016 =
  fun i loc ->
    (
# 228 "lib/stateParser.mly"
                              ( (loc,(Ty "ins_t", mk_instr_val i)) )
# 947 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_017 =
  fun i loc ->
    (
# 229 "lib/stateParser.mly"
                                   ( (loc,(Ty "ins_t", mk_instr_val i)) )
# 955 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_018 =
  fun loc typ v ->
    (
# 230 "lib/stateParser.mly"
                                              ( loc,(Atomic typ,v))
# 963 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_019 =
  fun loc typ v ->
    (
# 231 "lib/stateParser.mly"
                                       ( (loc,(Ty typ,v)))
# 971 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_020 =
  fun loc typ v ->
    (
# 233 "lib/stateParser.mly"
   ( (loc,(Ty typ,v)))
# 979 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_021 =
  fun _1 _2 ->
    (
# 235 "lib/stateParser.mly"
   ( let (t,sz) = _2 in
     let v0 = Constant.mk_replicate sz ParsedConstant.zero in
     (t,(TyArray (_1,sz),v0)) )
# 989 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_022 =
  fun _1 _2 _5 ->
    (
# 239 "lib/stateParser.mly"
   ( let (t,sz) = _2 and vs = _5 in
     if sz = List.length vs then
       let arr = (TyArray (_1,sz),Constant.mk_vec sz vs) in
       (t, arr)
     else
       Warn.user_error
         "Declared size of array %s does not match initial vakue size"
	 (dump_location t) )
# 1004 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_023 =
  fun () ->
    (
# 248 "lib/stateParser.mly"
                        ( raise Parsing.Parse_error )
# 1012 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_024 =
  fun loc typ ->
    (
# 249 "lib/stateParser.mly"
                             ( (loc,(Pointer typ,ParsedConstant.zero)))
# 1020 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_025 =
  fun loc typ v ->
    (
# 250 "lib/stateParser.mly"
                                                     ( (loc,(Pointer typ,v)))
# 1028 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_026 =
  fun loc ->
    (
# 251 "lib/stateParser.mly"
                    ( (loc,(TyDefPointer,ParsedConstant.zero)))
# 1036 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_027 =
  fun loc v ->
    (
# 252 "lib/stateParser.mly"
                                            ( (loc,(TyDefPointer,v)))
# 1044 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_028 =
  fun loc typ v ->
    (
# 254 "lib/stateParser.mly"
  ( (loc,(Ty typ, MiscParser.add_oa_if_none loc v)) )
# 1052 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_029 =
  fun loc v ->
    (
# 256 "lib/stateParser.mly"
  ( (loc,(Ty "pteval_t", MiscParser.add_oa_if_none loc v)) )
# 1060 "lib/stateParser.ml"
     : (MiscParser.state_atom))

let _menhir_action_030 =
  fun i loc ->
    (
# 408 "lib/stateParser.mly"
  (Atom (LV (Loc loc,(mk_instr_val i))))
# 1068 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_031 =
  fun _1 _3 ->
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1076 "lib/stateParser.ml"
     in
    (
# 409 "lib/stateParser.mly"
                        (Atom (LV (Loc _1,_3)))
# 1081 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_032 =
  fun _1 _3 ->
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1089 "lib/stateParser.ml"
     in
    (
# 409 "lib/stateParser.mly"
                        (Atom (LV (Loc _1,_3)))
# 1094 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_033 =
  fun loc v ->
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1102 "lib/stateParser.ml"
     in
    let loc = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1107 "lib/stateParser.ml"
     in
    (
# 411 "lib/stateParser.mly"
   (Atom (LV (Loc loc,v)))
# 1112 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_034 =
  fun loc v ->
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1120 "lib/stateParser.ml"
     in
    let loc = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1125 "lib/stateParser.ml"
     in
    (
# 411 "lib/stateParser.mly"
   (Atom (LV (Loc loc,v)))
# 1130 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_035 =
  fun _1 _3 ->
    (
# 412 "lib/stateParser.mly"
                           (Not (Atom (LV (Loc _1,_3))))
# 1138 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_036 =
  fun loc v ->
    let loc = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1146 "lib/stateParser.ml"
     in
    (
# 414 "lib/stateParser.mly"
   (Not (Atom (LV (Loc loc,v))))
# 1151 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_037 =
  fun loc v ->
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1159 "lib/stateParser.ml"
     in
    (
# 416 "lib/stateParser.mly"
  ( Atom (LV (Loc loc, MiscParser.add_oa_if_none loc v )) )
# 1164 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_038 =
  fun loc v ->
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1172 "lib/stateParser.ml"
     in
    (
# 416 "lib/stateParser.mly"
  ( Atom (LV (Loc loc, MiscParser.add_oa_if_none loc v )) )
# 1177 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_039 =
  fun loc v ->
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1185 "lib/stateParser.ml"
     in
    let loc = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1190 "lib/stateParser.ml"
     in
    (
# 418 "lib/stateParser.mly"
  ( Atom (LV (Loc loc, MiscParser.add_oa_if_none loc v)) )
# 1195 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_040 =
  fun loc v ->
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1203 "lib/stateParser.ml"
     in
    let loc = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1208 "lib/stateParser.ml"
     in
    (
# 418 "lib/stateParser.mly"
  ( Atom (LV (Loc loc, MiscParser.add_oa_if_none loc v)) )
# 1213 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_041 =
  fun _1 _4 ->
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1221 "lib/stateParser.ml"
     in
    (
# 421 "lib/stateParser.mly"
    ( let sz = List.length _4 in
      let vec = Constant.mk_vec sz _4 in
      Atom (LV (Loc _1,vec)) )
# 1228 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_042 =
  fun _1 _4 ->
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1236 "lib/stateParser.ml"
     in
    (
# 421 "lib/stateParser.mly"
    ( let sz = List.length _4 in
      let vec = Constant.mk_vec sz _4 in
      Atom (LV (Loc _1,vec)) )
# 1243 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_043 =
  fun _1 _4 ->
    (
# 425 "lib/stateParser.mly"
    ( let sz = List.length _4 in
      let vec = Constant.mk_vec sz _4 in
      Not (Atom (LV (Loc _1,vec))) )
# 1253 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_044 =
  fun _1 _3 ->
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1261 "lib/stateParser.ml"
     in
    (
# 428 "lib/stateParser.mly"
                        (Atom (LV (_1,_3)))
# 1266 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_045 =
  fun _1 _3 ->
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1274 "lib/stateParser.ml"
     in
    (
# 428 "lib/stateParser.mly"
                        (Atom (LV (_1,_3)))
# 1279 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_046 =
  fun _1 _3 ->
    (
# 429 "lib/stateParser.mly"
                           (Not (Atom (LV (_1,_3))))
# 1287 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_047 =
  fun _1 _3 ->
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1295 "lib/stateParser.ml"
     in
    (
# 430 "lib/stateParser.mly"
                                (Atom (LL (_1,_3)))
# 1300 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_048 =
  fun _1 _3 ->
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1308 "lib/stateParser.ml"
     in
    (
# 430 "lib/stateParser.mly"
                                (Atom (LL (_1,_3)))
# 1313 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_049 =
  fun loc loc_inlined1 ->
    let loc2 =
      let loc = loc_inlined1 in
      
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1323 "lib/stateParser.ml"
      
    in
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1329 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1334 "lib/stateParser.ml"
     in
    (
# 432 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1339 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_050 =
  fun loc loc_inlined1 ->
    let loc2 =
      let loc = loc_inlined1 in
      
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1349 "lib/stateParser.ml"
      
    in
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1355 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1360 "lib/stateParser.ml"
     in
    (
# 432 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1365 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_051 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 186 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 1375 "lib/stateParser.ml"
      
    in
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1381 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1386 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1391 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_052 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 187 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 1401 "lib/stateParser.ml"
      
    in
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1407 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1412 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1417 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_053 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 188 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 1427 "lib/stateParser.ml"
      
    in
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1433 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1438 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1443 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_054 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 189 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 1453 "lib/stateParser.ml"
      
    in
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1459 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1464 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1469 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_055 =
  fun _1_inlined2 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 190 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 1479 "lib/stateParser.ml"
      
    in
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1485 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1490 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1495 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_056 =
  fun _1_inlined2 _5 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 193 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 1505 "lib/stateParser.ml"
      
    in
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1511 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1516 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1521 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_057 =
  fun _1_inlined2 _5 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 195 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 1531 "lib/stateParser.ml"
      
    in
    let _2 = 
# 400 "lib/stateParser.mly"
        ( () )
# 1537 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1542 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1547 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_058 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 186 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 1557 "lib/stateParser.ml"
      
    in
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1563 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1568 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1573 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_059 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 187 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 1583 "lib/stateParser.ml"
      
    in
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1589 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1594 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1599 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_060 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 188 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 1609 "lib/stateParser.ml"
      
    in
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1615 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1620 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1625 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_061 =
  fun _1_inlined2 _3_inlined1 loc ->
    let loc2 =
      let (_3, _1) = (_3_inlined1, _1_inlined2) in
      
# 189 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 1635 "lib/stateParser.ml"
      
    in
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1641 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1646 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1651 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_062 =
  fun _1_inlined2 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 190 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 1661 "lib/stateParser.ml"
      
    in
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1667 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1672 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1677 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_063 =
  fun _1_inlined2 _5 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 193 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 1687 "lib/stateParser.ml"
      
    in
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1693 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1698 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1703 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_064 =
  fun _1_inlined2 _5 loc ->
    let loc2 =
      let _1 = _1_inlined2 in
      
# 195 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 1713 "lib/stateParser.ml"
      
    in
    let _2 = 
# 401 "lib/stateParser.mly"
             ( () )
# 1719 "lib/stateParser.ml"
     in
    let loc1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 1724 "lib/stateParser.ml"
     in
    (
# 434 "lib/stateParser.mly"
    ( Atom (LL (loc1,loc2)) )
# 1729 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_065 =
  fun _1 ->
    (
# 440 "lib/stateParser.mly"
        ( Atom (FF _1) )
# 1737 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_066 =
  fun fst rem ->
    (
# 287 "lib/stateParser.mly"
     ( String.concat ":" (fst::rem) )
# 1745 "lib/stateParser.ml"
     : (string))

let _menhir_action_067 =
  fun () ->
    (
# 352 "lib/stateParser.mly"
   ( ConstrGen.constr_true )
# 1753 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_068 =
  fun _2 ->
    (
# 354 "lib/stateParser.mly"
    (ForallStates _2)
# 1761 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_069 =
  fun _2 ->
    (
# 356 "lib/stateParser.mly"
    (ExistsState _2)
# 1769 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_070 =
  fun _3 ->
    (
# 358 "lib/stateParser.mly"
        ( NotExistsState _3 )
# 1777 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_071 =
  fun _2 ->
    (
# 360 "lib/stateParser.mly"
        ( ExistsState _2 )
# 1785 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_072 =
  fun _2 ->
    (
# 362 "lib/stateParser.mly"
    (ExistsState _2)
# 1793 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_073 =
  fun _2 ->
    (
# 364 "lib/stateParser.mly"
    ( ExistsState (Or _2) )
# 1801 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_074 =
  fun _1 _2 _3 ->
    (
# 323 "lib/stateParser.mly"
  ( let x = _1 in
    let f = _2 in
    let y,z = _3 in
    x,f,y,z )
# 1812 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.prop option * MiscParser.constr *
  (string * MiscParser.quantifier) list))

let _menhir_action_075 =
  fun lab ->
    (
# 289 "lib/stateParser.mly"
                          ( (lab,None,None) )
# 1822 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_076 =
  fun lab name ->
    (
# 291 "lib/stateParser.mly"
   ( if FaultType.is name then (lab,None,Some name)
     else (lab,Some (Constant.mk_sym name),None) )
# 1831 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_077 =
  fun fl lab ->
    (
# 293 "lib/stateParser.mly"
                                                 ( (lab,Some fl,None) )
# 1839 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_078 =
  fun ft lab ->
    (
# 294 "lib/stateParser.mly"
                                                       ( (lab,None,Some ft) )
# 1847 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_079 =
  fun fl ft lab ->
    (
# 297 "lib/stateParser.mly"
    ( (lab,Some fl,Some (String.concat ":" ft)) )
# 1855 "lib/stateParser.ml"
     : ((ParsedConstant.v, string) Fault.atom))

let _menhir_action_080 =
  fun name ->
    (
# 282 "lib/stateParser.mly"
            ( Constant.mk_sym name )
# 1863 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_081 =
  fun fl ->
    (
# 283 "lib/stateParser.mly"
                   ( fl )
# 1871 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_082 =
  fun name ->
    (
# 279 "lib/stateParser.mly"
                              ( (Constant.mk_sym_pte name) )
# 1879 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_083 =
  fun () ->
    (
# 315 "lib/stateParser.mly"
  ( None )
# 1887 "lib/stateParser.ml"
     : (MiscParser.prop option))

let _menhir_action_084 =
  fun _2 ->
    (
# 316 "lib/stateParser.mly"
              ( Some _2 )
# 1895 "lib/stateParser.ml"
     : (MiscParser.prop option))

let _menhir_action_085 =
  fun _1 ->
    (
# 345 "lib/stateParser.mly"
         ( _1 )
# 1903 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_086 =
  fun _1 ->
    (
# 346 "lib/stateParser.mly"
              ( _1 )
# 1911 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_087 =
  fun st ->
    (
# 115 "lib/stateParser.mly"
                        ( st )
# 1919 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_088 =
  fun () ->
    (
# 264 "lib/stateParser.mly"
  ([])
# 1927 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_089 =
  fun () ->
    (
# 265 "lib/stateParser.mly"
       ([])
# 1935 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_090 =
  fun _1 ->
    (
# 266 "lib/stateParser.mly"
            (_1::[])
# 1943 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_091 =
  fun _1 _3 ->
    (
# 267 "lib/stateParser.mly"
                                 (_1::_3)
# 1951 "lib/stateParser.ml"
     : (MiscParser.state))

let _menhir_action_092 =
  fun () ->
    (
# 222 "lib/stateParser.mly"
          ( None )
# 1959 "lib/stateParser.ml"
     : (string option))

let _menhir_action_093 =
  fun i ->
    (
# 223 "lib/stateParser.mly"
          ( Some i )
# 1967 "lib/stateParser.ml"
     : (string option))

let _menhir_action_094 =
  fun _1 ->
    (
# 339 "lib/stateParser.mly"
                    ( (_1,Require) )
# 1975 "lib/stateParser.ml"
     : (string * MiscParser.quantifier))

let _menhir_action_095 =
  fun _1 ->
    (
# 340 "lib/stateParser.mly"
                    ( (_1,Allow) )
# 1983 "lib/stateParser.ml"
     : (string * MiscParser.quantifier))

let _menhir_action_096 =
  fun _1 ->
    (
# 341 "lib/stateParser.mly"
                        ( (_1,Forbid) )
# 1991 "lib/stateParser.ml"
     : (string * MiscParser.quantifier))

let _menhir_action_097 =
  fun _1 ->
    (
# 334 "lib/stateParser.mly"
               ( [_1] )
# 1999 "lib/stateParser.ml"
     : ((string * MiscParser.quantifier) list))

let _menhir_action_098 =
  fun _1 ->
    (
# 335 "lib/stateParser.mly"
               ( [_1] )
# 2007 "lib/stateParser.ml"
     : ((string * MiscParser.quantifier) list))

let _menhir_action_099 =
  fun _1 _3 ->
    (
# 336 "lib/stateParser.mly"
                    ( _1 :: _3 )
# 2015 "lib/stateParser.ml"
     : ((string * MiscParser.quantifier) list))

let _menhir_action_100 =
  fun _1 ->
    (
# 384 "lib/stateParser.mly"
       ( (_1,None) )
# 2023 "lib/stateParser.ml"
     : (int * string option))

let _menhir_action_101 =
  fun _1 _3 ->
    (
# 385 "lib/stateParser.mly"
                  ( (_1,Some _3) )
# 2031 "lib/stateParser.ml"
     : (int * string option))

let _menhir_action_102 =
  fun loc ->
    (
# 214 "lib/stateParser.mly"
               ( loc )
# 2039 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_103 =
  fun loc ->
    (
# 215 "lib/stateParser.mly"
                                ( Location_global loc )
# 2047 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_104 =
  fun _1 _2 ->
    (
# 378 "lib/stateParser.mly"
                   ( _1,_2 )
# 2055 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.constr))

let _menhir_action_105 =
  fun _1 ->
    (
# 301 "lib/stateParser.mly"
           ( let a,t = _1 in LocationsItem.Loc (a,t) )
# 2064 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t))

let _menhir_action_106 =
  fun _1 ->
    (
# 302 "lib/stateParser.mly"
        (  Fault _1 )
# 2072 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t))

let _menhir_action_107 =
  fun _1 ->
    (
# 305 "lib/stateParser.mly"
           (_1::[])
# 2080 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_108 =
  fun _1 ->
    (
# 306 "lib/stateParser.mly"
                (_1::[])
# 2088 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_109 =
  fun _1 _3 ->
    (
# 307 "lib/stateParser.mly"
                           (_1::_3)
# 2096 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_110 =
  fun _1 _3 ->
    let _1 = 
# 186 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 2104 "lib/stateParser.ml"
     in
    (
# 210 "lib/stateParser.mly"
               ( _1 )
# 2109 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_111 =
  fun _1 _3 ->
    let _1 = 
# 187 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 2117 "lib/stateParser.ml"
     in
    (
# 210 "lib/stateParser.mly"
               ( _1 )
# 2122 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_112 =
  fun _1 _3 ->
    let _1 = 
# 188 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 2130 "lib/stateParser.ml"
     in
    (
# 210 "lib/stateParser.mly"
               ( _1 )
# 2135 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_113 =
  fun _1 _3 ->
    let _1 = 
# 189 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 2143 "lib/stateParser.ml"
     in
    (
# 210 "lib/stateParser.mly"
               ( _1 )
# 2148 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_114 =
  fun _1 ->
    let _1 = 
# 190 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 2156 "lib/stateParser.ml"
     in
    (
# 210 "lib/stateParser.mly"
               ( _1 )
# 2161 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_115 =
  fun _1 _5 ->
    let _1 = 
# 193 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 2169 "lib/stateParser.ml"
     in
    (
# 210 "lib/stateParser.mly"
               ( _1 )
# 2174 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_116 =
  fun _1 _5 ->
    let _1 = 
# 195 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 2182 "lib/stateParser.ml"
     in
    (
# 210 "lib/stateParser.mly"
               ( _1 )
# 2187 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_117 =
  fun _1 ->
    (
# 211 "lib/stateParser.mly"
                  ( Location_global _1 )
# 2195 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_118 =
  fun _1 _3 ->
    let _1 = 
# 186 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 2203 "lib/stateParser.ml"
     in
    (
# 201 "lib/stateParser.mly"
               ( _1 )
# 2208 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_119 =
  fun _1 _3 ->
    let _1 = 
# 187 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 2216 "lib/stateParser.ml"
     in
    (
# 201 "lib/stateParser.mly"
               ( _1 )
# 2221 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_120 =
  fun _1 _3 ->
    let _1 = 
# 188 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 2229 "lib/stateParser.ml"
     in
    (
# 201 "lib/stateParser.mly"
               ( _1 )
# 2234 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_121 =
  fun _1 _3 ->
    let _1 = 
# 189 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 2242 "lib/stateParser.ml"
     in
    (
# 201 "lib/stateParser.mly"
               ( _1 )
# 2247 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_122 =
  fun _1 ->
    let _1 = 
# 190 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 2255 "lib/stateParser.ml"
     in
    (
# 201 "lib/stateParser.mly"
               ( _1 )
# 2260 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_123 =
  fun _1 _5 ->
    let _1 = 
# 193 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 2268 "lib/stateParser.ml"
     in
    (
# 201 "lib/stateParser.mly"
               ( _1 )
# 2273 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_124 =
  fun _1 _5 ->
    let _1 = 
# 195 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 2281 "lib/stateParser.ml"
     in
    (
# 201 "lib/stateParser.mly"
               ( _1 )
# 2286 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_125 =
  fun _1_inlined1 _3 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 186 "lib/stateParser.mly"
                  (Location_reg (_1,_3))
# 2296 "lib/stateParser.ml"
      
    in
    (
# 202 "lib/stateParser.mly"
                    ( _2 )
# 2302 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_126 =
  fun _1_inlined1 _3 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 187 "lib/stateParser.mly"
                  (Location_reg (Misc.string_as_int _1,_3))
# 2312 "lib/stateParser.ml"
      
    in
    (
# 202 "lib/stateParser.mly"
                    ( _2 )
# 2318 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_127 =
  fun _1_inlined1 _3 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 188 "lib/stateParser.mly"
                       (Location_reg (_1,_3))
# 2328 "lib/stateParser.ml"
      
    in
    (
# 202 "lib/stateParser.mly"
                    ( _2 )
# 2334 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_128 =
  fun _1_inlined1 _3 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 189 "lib/stateParser.mly"
                       (Location_reg (Misc.string_as_int _1,_3))
# 2344 "lib/stateParser.ml"
      
    in
    (
# 202 "lib/stateParser.mly"
                    ( _2 )
# 2350 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_129 =
  fun _1_inlined1 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 190 "lib/stateParser.mly"
                  (Location_sreg _1 )
# 2360 "lib/stateParser.ml"
      
    in
    (
# 202 "lib/stateParser.mly"
                    ( _2 )
# 2366 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_130 =
  fun _1_inlined1 _5 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 193 "lib/stateParser.mly"
                  (Location_reg(Misc.string_as_int _1,_5))
# 2376 "lib/stateParser.ml"
      
    in
    (
# 202 "lib/stateParser.mly"
                    ( _2 )
# 2382 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_131 =
  fun _1_inlined1 _5 ->
    let _2 =
      let _1 = _1_inlined1 in
      
# 195 "lib/stateParser.mly"
                  (Location_reg(_1,_5))
# 2392 "lib/stateParser.ml"
      
    in
    (
# 202 "lib/stateParser.mly"
                    ( _2 )
# 2398 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_132 =
  fun _2 ->
    (
# 203 "lib/stateParser.mly"
            ( Location_global (Constant.mk_sym _2) )
# 2406 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_133 =
  fun loc ->
    let _1 = 
# 404 "lib/stateParser.mly"
                                ( Location_global loc )
# 2414 "lib/stateParser.ml"
     in
    (
# 204 "lib/stateParser.mly"
          ( _1 )
# 2419 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_134 =
  fun _1 ->
    (
# 123 "lib/stateParser.mly"
       ( Constant.mk_sym _1  )
# 2427 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_135 =
  fun _3 ->
    (
# 124 "lib/stateParser.mly"
                         ( Constant.mk_sym_pte  _3 )
# 2435 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_136 =
  fun _5 ->
    (
# 125 "lib/stateParser.mly"
                                           ( Constant.mk_sym_pte2 _5 )
# 2443 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_137 =
  fun _3 ->
    (
# 126 "lib/stateParser.mly"
                        ( Constant.mk_sym_pa _3 )
# 2451 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_138 =
  fun _1 _3 ->
    (
# 127 "lib/stateParser.mly"
                  ( mk_sym_tag _1 _3 )
# 2459 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_139 =
  fun id ->
    (
# 128 "lib/stateParser.mly"
                            ( mk_sym_tagloc_zero id )
# 2467 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_140 =
  fun id o ->
    (
# 129 "lib/stateParser.mly"
                                       ( mk_sym_tagloc id o )
# 2475 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_141 =
  fun _1 _3 _5 ->
    (
# 131 "lib/stateParser.mly"
                           (mk_sym_morello _1 _3 _5)
# 2483 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_142 =
  fun _1 _3 ->
    (
# 132 "lib/stateParser.mly"
                 ( mk_sym_morello "0" _1 _3 )
# 2491 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_143 =
  fun _3 ->
    (
# 310 "lib/stateParser.mly"
                                ( _3 )
# 2499 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_144 =
  fun () ->
    (
# 311 "lib/stateParser.mly"
                      ( [] )
# 2507 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_145 =
  fun () ->
    (
# 312 "lib/stateParser.mly"
  ( [] )
# 2515 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list))

let _menhir_action_146 =
  fun _1 ->
    (
# 393 "lib/stateParser.mly"
    ( let (loc,sz) = _1 in Deref (loc,sz) )
# 2523 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc))

let _menhir_action_147 =
  fun c ->
    (
# 349 "lib/stateParser.mly"
                 ( c )
# 2531 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_action_148 =
  fun f ->
    (
# 319 "lib/stateParser.mly"
               ( f )
# 2539 "lib/stateParser.ml"
     : (MiscParser.prop option))

let _menhir_action_149 =
  fun lc ->
    (
# 375 "lib/stateParser.mly"
                      ( lc )
# 2547 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) LocationsItem.t list *
  MiscParser.constr))

let _menhir_action_150 =
  fun loc ->
    (
# 207 "lib/stateParser.mly"
                   ( loc )
# 2556 "lib/stateParser.ml"
     : (MiscParser.location))

let _menhir_action_151 =
  fun _1 ->
    (
# 173 "lib/stateParser.mly"
               ( _1 )
# 2564 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_152 =
  fun _2 ->
    (
# 174 "lib/stateParser.mly"
              ( Tag _2 )
# 2572 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_153 =
  fun _1 ->
    (
# 181 "lib/stateParser.mly"
         ( _1 )
# 2580 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_154 =
  fun _1 _3 ->
    (
# 182 "lib/stateParser.mly"
                  ( mk_lab _1 _3 )
# 2588 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_155 =
  fun _1 _3 ->
    (
# 183 "lib/stateParser.mly"
                 ( mk_lab (Misc.string_as_int _1) _3 )
# 2596 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_156 =
  fun _1 _3 ->
    (
# 177 "lib/stateParser.mly"
                                 ( _1::_3 )
# 2604 "lib/stateParser.ml"
     : (ParsedConstant.v list))

let _menhir_action_157 =
  fun _1 ->
    (
# 178 "lib/stateParser.mly"
               ( [_1] )
# 2612 "lib/stateParser.ml"
     : (ParsedConstant.v list))

let _menhir_action_158 =
  fun _1 ->
    (
# 162 "lib/stateParser.mly"
       ( Concrete _1 )
# 2620 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_159 =
  fun _1 ->
    (
# 163 "lib/stateParser.mly"
        ( Concrete _1 )
# 2628 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_160 =
  fun _1 ->
    (
# 164 "lib/stateParser.mly"
                  ( _1 )
# 2636 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_161 =
  fun _1 _3 ->
    (
# 169 "lib/stateParser.mly"
                ( Concrete (_1 ^ ":" ^ _3) )
# 2644 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_162 =
  fun _1 _3 ->
    (
# 170 "lib/stateParser.mly"
                     ( mk_sym_with_index _1 _3 )
# 2652 "lib/stateParser.ml"
     : (ParsedConstant.v))

let _menhir_action_163 =
  fun _1 ->
    (
# 135 "lib/stateParser.mly"
       ( _1 )
# 2660 "lib/stateParser.ml"
     : (string))

let _menhir_action_164 =
  fun _1 ->
    (
# 136 "lib/stateParser.mly"
      ( _1 )
# 2668 "lib/stateParser.ml"
     : (string))

let _menhir_action_165 =
  fun _1 ->
    (
# 367 "lib/stateParser.mly"
          (  [ And _1 ] )
# 2676 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) ConstrGen.prop list))

let _menhir_action_166 =
  fun _1 _3 ->
    (
# 368 "lib/stateParser.mly"
                     ( And _1 :: _3 )
# 2684 "lib/stateParser.ml"
     : ((MiscParser.location, ParsedConstant.v, string) ConstrGen.prop list))

let _menhir_action_167 =
  fun () ->
    (
# 371 "lib/stateParser.mly"
                        ( [] )
# 2692 "lib/stateParser.ml"
     : (MiscParser.prop list))

let _menhir_action_168 =
  fun _1 _3 ->
    (
# 372 "lib/stateParser.mly"
                        ( _1 :: _3 )
# 2700 "lib/stateParser.ml"
     : (MiscParser.prop list))

let _menhir_action_169 =
  fun _1 ->
    (
# 329 "lib/stateParser.mly"
        ( _1,[] )
# 2708 "lib/stateParser.ml"
     : (MiscParser.constr * (string * MiscParser.quantifier) list))

let _menhir_action_170 =
  fun _1 _3 ->
    (
# 330 "lib/stateParser.mly"
                   ( _1,_3 )
# 2716 "lib/stateParser.ml"
     : (MiscParser.constr * (string * MiscParser.quantifier) list))

let _menhir_action_171 =
  fun name ->
    (
# 139 "lib/stateParser.mly"
            ( OutputAddress.parse name )
# 2724 "lib/stateParser.ml"
     : (OutputAddress.t))

let _menhir_action_172 =
  fun name ->
    (
# 140 "lib/stateParser.mly"
                             ( OutputAddress.PHY name )
# 2732 "lib/stateParser.ml"
     : (OutputAddress.t))

let _menhir_action_173 =
  fun name ->
    (
# 141 "lib/stateParser.mly"
                              ( OutputAddress.PTE name )
# 2740 "lib/stateParser.ml"
     : (OutputAddress.t))

let _menhir_action_174 =
  fun () ->
    (
# 444 "lib/stateParser.mly"
    (And [])
# 2748 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_175 =
  fun () ->
    (
# 446 "lib/stateParser.mly"
    (Or [])
# 2756 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_176 =
  fun _1 ->
    (
# 448 "lib/stateParser.mly"
    ( _1 )
# 2764 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_177 =
  fun _2 ->
    (
# 450 "lib/stateParser.mly"
    (Not _2)
# 2772 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_178 =
  fun _1 _3 ->
    (
# 452 "lib/stateParser.mly"
    (And [_1;_3])
# 2780 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_179 =
  fun _1 _3 ->
    (
# 454 "lib/stateParser.mly"
    (Or [_1;_3])
# 2788 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_180 =
  fun _1 _3 ->
    (
# 456 "lib/stateParser.mly"
    ( Implies (_1,_3) )
# 2796 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_181 =
  fun _2 ->
    (
# 458 "lib/stateParser.mly"
    ( _2 )
# 2804 "lib/stateParser.ml"
     : (MiscParser.prop))

let _menhir_action_182 =
  fun oa tail ->
    (
# 149 "lib/stateParser.mly"
    ( ParsedPteVal.add_oa oa tail )
# 2812 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_183 =
  fun key tail v ->
    (
# 151 "lib/stateParser.mly"
    ( ParsedPteVal.add_kv key v tail )
# 2820 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_184 =
  fun a tail ->
    (
# 153 "lib/stateParser.mly"
    ( ParsedPteVal.add_attr a tail )
# 2828 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_185 =
  fun attrs tail ->
    (
# 156 "lib/stateParser.mly"
    ( ParsedPteVal.add_attrs attrs tail )
# 2836 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_186 =
  fun () ->
    (
# 144 "lib/stateParser.mly"
  ( ParsedPteVal.empty )
# 2844 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_187 =
  fun pteval ->
    (
# 145 "lib/stateParser.mly"
                         ( pteval )
# 2852 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_188 =
  fun pteval ->
    (
# 159 "lib/stateParser.mly"
                             ( pteval )
# 2860 "lib/stateParser.ml"
     : (ParsedPteVal.t))

let _menhir_action_189 =
  fun _1 ->
    (
# 119 "lib/stateParser.mly"
             (  _1 )
# 2868 "lib/stateParser.ml"
     : (string))

let _menhir_action_190 =
  fun _1 ->
    (
# 120 "lib/stateParser.mly"
             (  _1 )
# 2876 "lib/stateParser.ml"
     : (string))

let _menhir_action_191 =
  fun _1 ->
    (
# 388 "lib/stateParser.mly"
           ( ConstrGen.Loc _1 )
# 2884 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc))

let _menhir_action_192 =
  fun _1 ->
    (
# 389 "lib/stateParser.mly"
           ( _1 )
# 2892 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc))

let _menhir_action_193 =
  fun _1 ->
    (
# 273 "lib/stateParser.mly"
       ( (_1, TyDef) )
# 2900 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc * TestType.t))

let _menhir_action_194 =
  fun _1 ->
    (
# 274 "lib/stateParser.mly"
            ( (_1, TyDefPointer) )
# 2908 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc * TestType.t))

let _menhir_action_195 =
  fun _1 _2 ->
    (
# 275 "lib/stateParser.mly"
            ( (_1, Ty _2) )
# 2916 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc * TestType.t))

let _menhir_action_196 =
  fun _1 _2 ->
    (
# 276 "lib/stateParser.mly"
                 ( (_1, Pointer _2) )
# 2924 "lib/stateParser.ml"
     : (MiscParser.location ConstrGen.rloc * TestType.t))

let _menhir_action_197 =
  fun x ->
    (
# 250 "<standard.mly>"
    ( [ x ] )
# 2932 "lib/stateParser.ml"
     : (string list))

let _menhir_action_198 =
  fun x xs ->
    (
# 253 "<standard.mly>"
    ( x :: xs )
# 2940 "lib/stateParser.ml"
     : (string list))

let _menhir_action_199 =
  fun x ->
    (
# 250 "<standard.mly>"
    ( [ x ] )
# 2948 "lib/stateParser.ml"
     : (string list))

let _menhir_action_200 =
  fun x xs ->
    (
# 253 "<standard.mly>"
    ( x :: xs )
# 2956 "lib/stateParser.ml"
     : (string list))

let _menhir_action_201 =
  fun _2 ->
    (
# 381 "lib/stateParser.mly"
                       ( _2 )
# 2964 "lib/stateParser.ml"
     : (MiscParser.constr))

let _menhir_print_token : token -> string =
  fun _tok ->
    match _tok with
    | AMPER ->
        "AMPER"
    | AND ->
        "AND"
    | ATOMIC ->
        "ATOMIC"
    | ATOMICINIT ->
        "ATOMICINIT"
    | ATTRS ->
        "ATTRS"
    | COLON ->
        "COLON"
    | COMMA ->
        "COMMA"
    | DOLLARNAME _ ->
        "DOLLARNAME"
    | EOF ->
        "EOF"
    | EQUAL ->
        "EQUAL"
    | EQUALEQUAL ->
        "EQUALEQUAL"
    | EXISTS ->
        "EXISTS"
    | FALSE ->
        "FALSE"
    | FAULT ->
        "FAULT"
    | FILTER ->
        "FILTER"
    | FINAL ->
        "FINAL"
    | FORALL ->
        "FORALL"
    | IMPLIES ->
        "IMPLIES"
    | INSTR _ ->
        "INSTR"
    | LBRK ->
        "LBRK"
    | LCURLY ->
        "LCURLY"
    | LOCATIONS ->
        "LOCATIONS"
    | LPAR ->
        "LPAR"
    | NAME _ ->
        "NAME"
    | NOT ->
        "NOT"
    | NOTEQUAL ->
        "NOTEQUAL"
    | NUM _ ->
        "NUM"
    | OBSERVED ->
        "OBSERVED"
    | OR ->
        "OR"
    | PLUS ->
        "PLUS"
    | PROC _ ->
        "PROC"
    | PTX_REG_DEC ->
        "PTX_REG_DEC"
    | PTX_REG_TYPE _ ->
        "PTX_REG_TYPE"
    | RBRK ->
        "RBRK"
    | RCURLY ->
        "RCURLY"
    | RPAR ->
        "RPAR"
    | SEMI ->
        "SEMI"
    | STAR ->
        "STAR"
    | SYMB_REG _ ->
        "SYMB_REG"
    | TOKAND ->
        "TOKAND"
    | TOK_NOP ->
        "TOK_NOP"
    | TOK_OA ->
        "TOK_OA"
    | TOK_PA ->
        "TOK_PA"
    | TOK_PTE ->
        "TOK_PTE"
    | TOK_TAG ->
        "TOK_TAG"
    | TRUE ->
        "TRUE"
    | VALUE _ ->
        "VALUE"
    | WITH ->
        "WITH"

let _menhir_fail : unit -> 'a =
  fun () ->
    Printf.eprintf "Internal failure -- please contact the parser generator's developers.\n%!";
    assert false

include struct
  
  [@@@ocaml.warning "-4-37"]
  
  let _menhir_run_392 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_location =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let loc = _v in
          let _v = _menhir_action_150 loc in
          MenhirBox_main_location _v
      | _ ->
          _eRR ()
  
  let _menhir_run_402 : type  ttv_stack. (ttv_stack, _menhir_box_skip_loc_constr) _menhir_cell1_locations -> _ -> _ -> _menhir_box_skip_loc_constr =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let MenhirCell1_locations (_menhir_stack, _, _) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_201 _2 in
          MenhirBox_skip_loc_constr _v
      | _ ->
          _eRR ()
  
  let _menhir_run_387 : type  ttv_stack. (ttv_stack, _menhir_box_main_loc_constr) _menhir_cell1_locations -> _ -> _ -> _menhir_box_main_loc_constr =
    fun _menhir_stack _v _tok ->
      let MenhirCell1_locations (_menhir_stack, _, _1) = _menhir_stack in
      let _2 = _v in
      let _v = _menhir_action_104 _1 _2 in
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let lc = _v in
          let _v = _menhir_action_149 lc in
          MenhirBox_main_loc_constr _v
      | _ ->
          _eRR ()
  
  let _menhir_run_378 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_constr =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let c = _v in
          let _v = _menhir_action_147 c in
          MenhirBox_main_constr _v
      | _ ->
          _eRR ()
  
  let _menhir_goto_old_constraints : type  ttv_stack. ((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter -> _ -> _menhir_box_constraints =
    fun _menhir_stack _v ->
      let MenhirCell1_filter (_menhir_stack, _, _2) = _menhir_stack in
      let MenhirCell1_locations (_menhir_stack, _, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_074 _1 _2 _3 in
      MenhirBox_constraints _v
  
  let _menhir_run_307 : type  ttv_stack. (((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter, _menhir_box_constraints) _menhir_cell1_final -> _ -> _menhir_box_constraints =
    fun _menhir_stack _v ->
      let MenhirCell1_final (_menhir_stack, _, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_170 _1 _3 in
      _menhir_goto_old_constraints _menhir_stack _v
  
  let rec _menhir_goto_kinds : type  ttv_stack. ttv_stack -> _ -> (ttv_stack, _menhir_box_constraints) _menhir_state -> _menhir_box_constraints =
    fun _menhir_stack _v _menhir_s ->
      match _menhir_s with
      | MenhirState309 ->
          _menhir_run_310 _menhir_stack _v
      | MenhirState300 ->
          _menhir_run_307 _menhir_stack _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_310 : type  ttv_stack. (ttv_stack, _menhir_box_constraints) _menhir_cell1_kind -> _ -> _menhir_box_constraints =
    fun _menhir_stack _v ->
      let MenhirCell1_kind (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_099 _1 _3 in
      _menhir_goto_kinds _menhir_stack _v _menhir_s
  
  let rec _menhir_run_301 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_constraints) _menhir_state -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NOT ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | EXISTS ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let _1 = _v in
                  let _v = _menhir_action_096 _1 in
                  _menhir_goto_kind _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | _ ->
                  _eRR ())
          | FORALL ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _1 = _v in
              let _v = _menhir_action_094 _1 in
              _menhir_goto_kind _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | EXISTS ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _1 = _v in
              let _v = _menhir_action_095 _1 in
              _menhir_goto_kind _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_kind : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v_0 ->
              let _menhir_stack = MenhirCell1_kind (_menhir_stack, _menhir_s, _v) in
              _menhir_run_301 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState309
          | EOF ->
              let _1 = _v in
              let _v = _menhir_action_098 _1 in
              _menhir_goto_kinds _menhir_stack _v _menhir_s
          | _ ->
              _eRR ())
      | EOF ->
          let _1 = _v in
          let _v = _menhir_action_097 _1 in
          _menhir_goto_kinds _menhir_stack _v _menhir_s
      | _ ->
          _eRR ()
  
  let _menhir_goto_final : type  ttv_stack. (((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | WITH ->
          let _menhir_stack = MenhirCell1_final (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState300 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              _menhir_run_301 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | EOF ->
          let _1 = _v in
          let _v = _menhir_action_169 _1 in
          _menhir_goto_old_constraints _menhir_stack _v
      | _ ->
          _eRR ()
  
  let _menhir_run_311 : type  ttv_stack. (((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations, _menhir_box_constraints) _menhir_cell1_filter as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _1 = _v in
          let _v = _menhir_action_086 _1 in
          _menhir_goto_final _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | EOF | WITH ->
          let _1 = _v in
          let _v = _menhir_action_085 _1 in
          _menhir_goto_final _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  let _menhir_goto_constr : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState401 ->
          _menhir_run_402 _menhir_stack _v _tok
      | MenhirState386 ->
          _menhir_run_387 _menhir_stack _v _tok
      | MenhirState376 ->
          _menhir_run_378 _menhir_stack _v _tok
      | MenhirState276 ->
          _menhir_run_311 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  let _menhir_run_284 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_OBSERVED -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_OBSERVED (_menhir_stack, _menhir_s) = _menhir_stack in
      let _2 = _v in
      let _v = _menhir_action_073 _2 in
      _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  let rec _menhir_goto_obs : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState277 ->
          _menhir_run_284 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState279 ->
          _menhir_run_280 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_280 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_obsone -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_obsone (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_166 _1 _3 in
      _menhir_goto_obs _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  let _menhir_run_382 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_filter =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let f = _v in
          let _v = _menhir_action_148 f in
          MenhirBox_main_filter _v
      | _ ->
          _eRR ()
  
  let _menhir_run_398 : type  ttv_stack. ttv_stack -> _ -> _menhir_box_pteval =
    fun _menhir_stack _v ->
      MenhirBox_pteval _v
  
  let _menhir_run_369 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _v _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF ->
          let st = _v in
          let _v = _menhir_action_087 st in
          MenhirBox_init _v
      | _ ->
          _eRR ()
  
  let rec _menhir_goto_init_semi_list : type  ttv_stack. ttv_stack -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState373 ->
          _menhir_run_374 _menhir_stack _v _tok
      | MenhirState314 ->
          _menhir_run_369 _menhir_stack _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_374 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_atom_init -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _v _tok ->
      let MenhirCell1_atom_init (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_091 _1 _3 in
      _menhir_goto_init_semi_list _menhir_stack _v _menhir_s _tok
  
  let _menhir_run_325 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_089 () in
      _menhir_goto_init_semi_list _menhir_stack _v _menhir_s _tok
  
  let rec _menhir_run_003 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let id = _v in
                  let _v = _menhir_action_139 id in
                  _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PLUS ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NUM _v_0 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let (id, o) = (_v, _v_0) in
                          let _v = _menhir_action_140 id o in
                          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_location_global : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState332 ->
          _menhir_run_317 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState316 ->
          _menhir_run_317 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState254 ->
          _menhir_run_255 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState190 ->
          _menhir_run_191 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState169 ->
          _menhir_run_170 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState358 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState354 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState351 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState345 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState335 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState337 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState330 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState323 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState260 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState262 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState218 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState251 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState210 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState211 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState213 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState207 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState205 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState203 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState174 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState119 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState105 ->
          _menhir_run_118 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState098 ->
          _menhir_run_103 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState390 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState373 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState314 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState349 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState326 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState327 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState315 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState295 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState293 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState291 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState288 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState286 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState277 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState282 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState279 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState272 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_084 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_317 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_LBRK -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_103 loc in
          _menhir_goto_left_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_left_loc : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState373 ->
          _menhir_run_357 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState314 ->
          _menhir_run_357 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState349 ->
          _menhir_run_350 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState326 ->
          _menhir_run_334 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState327 ->
          _menhir_run_328 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState315 ->
          _menhir_run_320 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_357 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v_0 ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState358
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState358
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState358
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState358
          | TOK_NOP ->
              _menhir_run_261 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState358
          | PROC _v_1 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | COLON ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v_2 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      let (_1, _3) = (_v_1, _v_2) in
                      let _v = _menhir_action_154 _1 _3 in
                      _menhir_goto_maybev_label _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NUM _v_4 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | COLON ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NUM _v_5 ->
                      let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState358, _v_4) in
                      _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5
                  | NAME _v_6 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | COLON ->
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState358, _v_4) in
                          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, MenhirState363, _v_6) in
                          _menhir_run_041 _menhir_stack _menhir_lexbuf _menhir_lexer
                      | EOF | SEMI ->
                          let (_1, _3) = (_v_4, _v_6) in
                          let _v = _menhir_action_155 _1 _3 in
                          _menhir_goto_maybev_label _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | EOF | SEMI ->
                  let _v_8 =
                    let _1 = _v_4 in
                    _menhir_action_158 _1
                  in
                  _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v_8 MenhirState358 _tok
              | _ ->
                  _eRR ())
          | NAME _v_9 ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_9 MenhirState358
          | LPAR ->
              _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState358
          | INSTR _v_10 ->
              _menhir_run_265 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState358
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState358
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_106 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_159 _1 in
      _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_maybev_notag : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState345 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState262 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState251 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState213 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState211 ->
          _menhir_run_212 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState358 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState354 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState351 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState335 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState337 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState330 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState323 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState260 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState218 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState210 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState207 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState205 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState203 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState174 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState119 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState105 ->
          _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_212 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          let _menhir_stack = MenhirCell1_maybev_notag (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState213 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | RCURLY ->
          let _1 = _v in
          let _v = _menhir_action_157 _1 in
          _menhir_goto_maybev_list _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_010 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_PTE ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | RPAR ->
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              let _5 = _v in
                              let _v = _menhir_action_136 _5 in
                              _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                          | _ ->
                              _eRR ())
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let _3 = _v in
                  let _v = _menhir_action_135 _3 in
                  _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_019 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let _3 = _v in
                  let _v = _menhir_action_137 _3 in
                  _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_107 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState108 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NUM _v ->
              _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v
          | NAME _v ->
              _menhir_run_101 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | AND | COMMA | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RCURLY | RPAR | SEMI | WITH ->
          let _1 = _v in
          let _v = _menhir_action_158 _1 in
          _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_109 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_161 _1 _3 in
      _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_101 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          _menhir_run_041 _menhir_stack _menhir_lexbuf _menhir_lexer
      | _ ->
          _eRR ()
  
  and _menhir_run_041 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_NAME -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NUM _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_NAME (_menhir_stack, _, _3) = _menhir_stack in
          let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _5 = _v in
          let _v = _menhir_action_141 _1 _3 _5 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_110 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NUM _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RBRK ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let (_1, _3) = (_v, _v_0) in
                  let _v = _menhir_action_162 _1 _3 in
                  _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_048 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND | COMMA | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RCURLY | RPAR | SEMI | WITH ->
          let _1 = _v in
          let _v = _menhir_action_134 _1 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_048 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NAME -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NUM _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_142 _1 _3 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | NAME _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_138 _1 _3 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_maybev_list : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      match _menhir_s with
      | MenhirState345 ->
          _menhir_run_346 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState262 ->
          _menhir_run_263 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState251 ->
          _menhir_run_252 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState211 ->
          _menhir_run_215 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState213 ->
          _menhir_run_214 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_346 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_arrayspec -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_arrayspec (_menhir_stack, _, _2) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_022 _1 _2 _5 in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_atom_init : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _menhir_stack = MenhirCell1_atom_init (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState373
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState373
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState373
          | SYMB_REG _v_0 ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState373
          | STAR ->
              _menhir_run_315 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState373
          | SEMI ->
              _menhir_run_325 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState373
          | PROC _v_1 ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState373
          | NUM _v_2 ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState373
          | NAME _v_3 ->
              _menhir_run_326 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState373
          | LBRK ->
              _menhir_run_316 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState373
          | ATOMIC ->
              _menhir_run_348 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState373
          | EOF ->
              let _v_4 = _menhir_action_088 () in
              _menhir_run_374 _menhir_stack _v_4 _tok
          | _ ->
              _eRR ())
      | EOF ->
          let _1 = _v in
          let _v = _menhir_action_090 _1 in
          _menhir_goto_init_semi_list _menhir_stack _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_023 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_114 _1 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_location : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState390 ->
          _menhir_run_392 _menhir_stack _v _tok
      | MenhirState373 ->
          _menhir_run_356 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState314 ->
          _menhir_run_356 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState349 ->
          _menhir_run_319 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState326 ->
          _menhir_run_319 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState327 ->
          _menhir_run_319 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState315 ->
          _menhir_run_319 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState295 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState293 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState291 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState288 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState286 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState277 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState282 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState279 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState272 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_209 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_085 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_085 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_356 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EOF | SEMI ->
          let _1 = _v in
          let _v = _menhir_action_011 _1 in
          _menhir_goto_atom _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | EQUAL ->
          let loc = _v in
          let _v = _menhir_action_102 loc in
          _menhir_goto_left_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_atom : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_013 _1 in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_319 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let loc = _v in
      let _v = _menhir_action_102 loc in
      _menhir_goto_left_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_209 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_location (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | NOTEQUAL ->
          let _menhir_s = MenhirState210 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LCURLY ->
              let _menhir_stack = MenhirCell1_LCURLY (_menhir_stack, _menhir_s) in
              let _menhir_s = MenhirState211 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EQUALEQUAL ->
          let _menhir_s = MenhirState218 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | SYMB_REG _v ->
              _menhir_run_219 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | STAR ->
              _menhir_run_220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | PROC _v ->
              _menhir_run_237 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NUM _v ->
              _menhir_run_244 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LPAR ->
              _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | LCURLY ->
              let _menhir_stack = MenhirCell1_LCURLY (_menhir_stack, _menhir_s) in
              let _menhir_s = MenhirState251 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | LBRK ->
              _menhir_run_254 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EQUAL ->
          let _menhir_s = MenhirState260 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_NOP ->
              _menhir_run_261 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | SYMB_REG _v ->
              _menhir_run_219 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | STAR ->
              _menhir_run_220 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | PROC _v ->
              _menhir_run_237 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NUM _v ->
              _menhir_run_244 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LPAR ->
              _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | LCURLY ->
              let _menhir_stack = MenhirCell1_LCURLY (_menhir_stack, _menhir_s) in
              let _menhir_s = MenhirState262 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | LBRK ->
              _menhir_run_254 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | INSTR _v ->
              _menhir_run_265 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_114 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NAME _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _2 = _v in
          let _v = _menhir_action_152 _2 in
          _menhir_goto_maybev _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_maybev : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState358 ->
          _menhir_run_367 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState354 ->
          _menhir_run_355 _menhir_stack _menhir_lexbuf _menhir_lexer _tok
      | MenhirState351 ->
          _menhir_run_352 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState335 ->
          _menhir_run_341 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState337 ->
          _menhir_run_338 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState330 ->
          _menhir_run_331 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState323 ->
          _menhir_run_324 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState260 ->
          _menhir_run_267 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState218 ->
          _menhir_run_258 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState210 ->
          _menhir_run_217 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState207 ->
          _menhir_run_208 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState205 ->
          _menhir_run_206 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState203 ->
          _menhir_run_204 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState174 ->
          _menhir_run_194 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState119 ->
          _menhir_run_173 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState105 ->
          _menhir_run_117 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_367 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let _1 = _v in
      let _v = _menhir_action_153 _1 in
      _menhir_goto_maybev_label _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
  
  and _menhir_goto_maybev_label : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_012 _1 _3 in
      _menhir_goto_atom _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_355 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_locindex -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _tok ->
      let MenhirCell1_locindex (_menhir_stack, _menhir_s, _) = _menhir_stack in
      let _v = _menhir_action_023 () in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_352 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_ATOMIC _menhir_cell0_NAME, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell0_NAME (_menhir_stack, typ) = _menhir_stack in
      let MenhirCell1_ATOMIC (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_018 loc typ v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_341 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_019 loc typ v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_338 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_ATOMICINIT -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_ATOMICINIT (_menhir_stack, _) = _menhir_stack in
          let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
          let v = _v in
          let _v = _menhir_action_020 loc typ v in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_331 : type  ttv_stack. ((((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_amperopt -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_amperopt (_menhir_stack, _, _) = _menhir_stack in
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_025 loc typ v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_324 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc, _menhir_box_init) _menhir_cell1_amperopt -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_amperopt (_menhir_stack, _, _) = _menhir_stack in
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_027 loc v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_267 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_031 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_atom_prop : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState277 ->
          _menhir_run_281 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState282 ->
          _menhir_run_281 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState279 ->
          _menhir_run_281 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState295 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState293 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState291 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState288 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState286 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState272 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_271 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_281 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_atom_prop (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState282
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState282
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState282
          | SYMB_REG _v_0 ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState282
          | PROC _v_1 ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState282
          | NUM _v_2 ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState282
          | NAME _v_3 ->
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState282
          | LBRK ->
              _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState282
          | FAULT ->
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState282
          | EOF | SEMI | TOKAND | WITH ->
              let _v_4 = _menhir_action_167 () in
              _menhir_run_283 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 _tok
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_025 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | SYMB_REG _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _3) = (_v, _v_0) in
              let _v = _menhir_action_112 _1 _3 in
              _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PTX_REG_DEC ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState026) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | PTX_REG_TYPE _v ->
                  let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                  let _menhir_s = MenhirState029 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | DOLLARNAME _v ->
                      _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v_4 ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState026
          | DOLLARNAME _v_5 ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState026
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_030 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_189 _1 in
      _menhir_goto_reg _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_reg : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState245 ->
          _menhir_run_250 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState248 ->
          _menhir_run_249 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState238 ->
          _menhir_run_243 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState241 ->
          _menhir_run_242 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState230 ->
          _menhir_run_235 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState233 ->
          _menhir_run_234 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState223 ->
          _menhir_run_228 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState226 ->
          _menhir_run_227 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState184 ->
          _menhir_run_189 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState187 ->
          _menhir_run_188 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState177 ->
          _menhir_run_182 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState180 ->
          _menhir_run_181 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState129 ->
          _menhir_run_134 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState132 ->
          _menhir_run_133 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState122 ->
          _menhir_run_127 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState125 ->
          _menhir_run_126 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState035 ->
          _menhir_run_043 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState038 ->
          _menhir_run_039 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState026 ->
          _menhir_run_033 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState029 ->
          _menhir_run_032 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_250 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_119 _1 _3 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_location_deref : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState260 ->
          _menhir_run_268 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState218 ->
          _menhir_run_259 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_268 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_047 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_259 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_048 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_249 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_123 _1 _5 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_243 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_PROC (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_118 _1 _3 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_242 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PROC (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_124 _1 _5 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_235 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_STAR _menhir_cell0_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_NUM (_menhir_stack, _1_inlined1) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_126 _1_inlined1 _3 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_234 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_STAR _menhir_cell0_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell0_NUM (_menhir_stack, _1_inlined1) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_130 _1_inlined1 _5 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_228 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_STAR _menhir_cell0_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PROC (_menhir_stack, _1_inlined1) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_125 _1_inlined1 _3 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_227 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_STAR _menhir_cell0_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell0_PROC (_menhir_stack, _1_inlined1) = _menhir_stack in
      let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_131 _1_inlined1 _5 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_189 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NUM (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3_inlined1 = _v in
      let _v = _menhir_action_052 _1_inlined2 _3_inlined1 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_188 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NUM (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_056 _1_inlined2 _5 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_182 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_PROC (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3_inlined1 = _v in
      let _v = _menhir_action_051 _1_inlined2 _3_inlined1 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_181 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PROC (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_057 _1_inlined2 _5 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_134 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NUM (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3_inlined1 = _v in
      let _v = _menhir_action_059 _1_inlined2 _3_inlined1 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_133 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NUM (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_063 _1_inlined2 _5 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_127 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_PROC (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3_inlined1 = _v in
      let _v = _menhir_action_058 _1_inlined2 _3_inlined1 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_126 : type  ttv_stack ttv_result. ((((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PROC (_menhir_stack, _, _1_inlined2) = _menhir_stack in
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_064 _1_inlined2 _5 loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_043 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NUM -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_111 _1 _3 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_039 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NUM (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_115 _1 _5 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_033 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_PROC -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_PROC (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_110 _1 _3 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_032 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_PROC, ttv_result) _menhir_cell1_PTX_REG_DEC _menhir_cell0_PTX_REG_TYPE -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_PTX_REG_TYPE (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PTX_REG_DEC (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_PROC (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _5 = _v in
      let _v = _menhir_action_116 _1 _5 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_031 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_190 _1 in
      _menhir_goto_reg _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_034 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | SYMB_REG _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _3) = (_v, _v_0) in
              let _v = _menhir_action_113 _1 _3 in
              _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PTX_REG_DEC ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState035) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | PTX_REG_TYPE _v ->
                  let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                  let _menhir_s = MenhirState038 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | DOLLARNAME _v ->
                      _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v_4 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_040 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState035
          | DOLLARNAME _v_5 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState035
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_040 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NUM as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_041 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND | EOF | EQUAL | EQUALEQUAL | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NAME _ | NOT | NOTEQUAL | OBSERVED | OR | RBRK | RPAR | SEMI | STAR | WITH ->
          let _1 = _v in
          let _v = _menhir_action_189 _1 in
          _menhir_goto_reg _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_044 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LBRK ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, MenhirState044) in
          let _menhir_s = MenhirState045 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NUM _v ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RBRK ->
                  _menhir_run_047 _menhir_stack _menhir_lexbuf _menhir_lexer
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_048 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EQUAL | EQUALEQUAL | NAME _ | NOTEQUAL | RBRK | SEMI | STAR ->
          let _1 = _v in
          let _v = _menhir_action_134 _1 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_047 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_NAME, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_NUM -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_NUM (_menhir_stack, _, _3) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _v = _menhir_action_010 _1 _3 in
      _menhir_goto_arrayspec _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_arrayspec : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState326 ->
          _menhir_run_343 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState314 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState373 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState295 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState293 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState291 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState288 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState286 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState277 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState279 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState282 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState272 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_092 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_343 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_arrayspec (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | LCURLY ->
              let _menhir_s = MenhirState345 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_021 _1 _2 in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_092 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_146 _1 in
      _menhir_goto_locindex _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_locindex : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState373 ->
          _menhir_run_353 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState314 ->
          _menhir_run_353 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState295 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState293 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState291 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState288 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState286 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState277 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState282 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState279 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState272 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_202 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_083 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_083 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_353 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locindex (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_s = MenhirState354 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_202 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locindex (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | NOTEQUAL ->
          let _menhir_s = MenhirState203 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EQUALEQUAL ->
          let _menhir_s = MenhirState205 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EQUAL ->
          let _menhir_s = MenhirState207 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_083 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_192 _1 in
      _menhir_goto_rloc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_rloc : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | STAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _1 = _v in
          let _v = _menhir_action_194 _1 in
          _menhir_goto_rloc_typ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | NAME _v_0 ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | STAR ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _2) = (_v, _v_0) in
              let _v = _menhir_action_196 _1 _2 in
              _menhir_goto_rloc_typ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | RBRK | SEMI ->
              let (_1, _2) = (_v, _v_0) in
              let _v = _menhir_action_195 _1 _2 in
              _menhir_goto_rloc_typ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | _ ->
              _eRR ())
      | RBRK | SEMI ->
          let _1 = _v in
          let _v = _menhir_action_193 _1 in
          _menhir_goto_rloc_typ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_rloc_typ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_105 _1 in
      _menhir_goto_loc_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_loc_item : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | SEMI ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState089
          | TOK_PTE ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState089
          | TOK_PA ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState089
          | SYMB_REG _v_0 ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState089
          | PROC _v_1 ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState089
          | NUM _v_2 ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState089
          | NAME _v_3 ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState089
          | FAULT ->
              let _menhir_stack = MenhirCell1_loc_item (_menhir_stack, _menhir_s, _v) in
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState089
          | RBRK ->
              let _1 = _v in
              let _v = _menhir_action_108 _1 in
              _menhir_goto_loc_items _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | RBRK ->
          let _1 = _v in
          let _v = _menhir_action_107 _1 in
          _menhir_goto_loc_items _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_051 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_FAULT (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | PROC _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | COLON ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v_0 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      let (_1, _3) = (_v, _v_0) in
                      let _v = _menhir_action_101 _1 _3 in
                      _menhir_goto_lbl _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                  | _ ->
                      _eRR ())
              | COMMA | RPAR ->
                  let _1 = _v in
                  let _v = _menhir_action_100 _1 in
                  _menhir_goto_lbl _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_lbl : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_FAULT -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
          let lab = _v in
          let _v = _menhir_action_075 lab in
          _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | COMMA ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_PTE ->
              let _menhir_stack = MenhirCell0_lbl (_menhir_stack, _v) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v_0 ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let name = _v_0 in
                          let _v = _menhir_action_082 name in
                          (match (_tok : MenhirBasics.token) with
                          | RPAR ->
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              let MenhirCell0_lbl (_menhir_stack, lab) = _menhir_stack in
                              let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
                              let fl = _v in
                              let _v = _menhir_action_077 fl lab in
                              _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                          | COMMA ->
                              let fl = _v in
                              let _v = _menhir_action_081 fl in
                              _menhir_goto_fault_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v
                          | _ ->
                              _eRR ())
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v_2 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
                  let (name, lab) = (_v_2, _v) in
                  let _v = _menhir_action_076 lab name in
                  _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | COLON ->
                  let _menhir_stack = MenhirCell0_lbl (_menhir_stack, _v) in
                  let _menhir_stack = MenhirCell0_NAME (_menhir_stack, _v_2) in
                  let _menhir_s = MenhirState065 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_066 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | COMMA ->
                  let _menhir_stack = MenhirCell0_lbl (_menhir_stack, _v) in
                  let name = _v_2 in
                  let _v = _menhir_action_080 name in
                  _menhir_goto_fault_loc _menhir_stack _menhir_lexbuf _menhir_lexer _v
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_fault : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState295 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState293 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState291 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState288 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState286 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState277 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState282 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState279 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState272 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_270 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState002 ->
          _menhir_run_091 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState089 ->
          _menhir_run_091 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_270 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_065 _1 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_091 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_106 _1 in
      _menhir_goto_loc_item _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_fault_loc : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_FAULT _menhir_cell0_lbl -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _menhir_stack = MenhirCell0_fault_loc (_menhir_stack, _v) in
      let _menhir_s = MenhirState073 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NAME _v ->
          _menhir_run_066 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_066 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState067 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              _menhir_run_066 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | RPAR ->
          let x = _v in
          let _v = _menhir_action_197 x in
          _menhir_goto_separated_nonempty_list_COLON_NAME_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_goto_separated_nonempty_list_COLON_NAME_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      match _menhir_s with
      | MenhirState073 ->
          _menhir_run_074 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState065 ->
          _menhir_run_069 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState067 ->
          _menhir_run_068 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_074 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_FAULT _menhir_cell0_lbl _menhir_cell0_fault_loc -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell0_fault_loc (_menhir_stack, fl) = _menhir_stack in
      let MenhirCell0_lbl (_menhir_stack, lab) = _menhir_stack in
      let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
      let ft = _v in
      let _v = _menhir_action_079 fl ft lab in
      _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_069 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_FAULT _menhir_cell0_lbl _menhir_cell0_NAME -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell0_NAME (_menhir_stack, fst) = _menhir_stack in
      let rem = _v in
      let _v = _menhir_action_066 fst rem in
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell0_lbl (_menhir_stack, lab) = _menhir_stack in
      let MenhirCell1_FAULT (_menhir_stack, _menhir_s) = _menhir_stack in
      let ft = _v in
      let _v = _menhir_action_078 ft lab in
      _menhir_goto_fault _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_068 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NAME -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, x) = _menhir_stack in
      let xs = _v in
      let _v = _menhir_action_198 x xs in
      _menhir_goto_separated_nonempty_list_COLON_NAME_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
  
  and _menhir_goto_loc_items : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      match _menhir_s with
      | MenhirState089 ->
          _menhir_run_090 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState002 ->
          _menhir_run_086 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_090 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_loc_item -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell1_loc_item (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_109 _1 _3 in
      _menhir_goto_loc_items _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
  
  and _menhir_run_086 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LOCATIONS -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_143 _3 in
      _menhir_goto_locations _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_locations : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState399 ->
          _menhir_run_401 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState384 ->
          _menhir_run_386 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState000 ->
          _menhir_run_093 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_401 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_skip_loc_constr) _menhir_state -> _ -> _menhir_box_skip_loc_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locations (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | OBSERVED ->
          _menhir_run_277 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState401
      | NOT ->
          _menhir_run_285 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState401
      | LPAR ->
          _menhir_run_288 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState401
      | FORALL ->
          _menhir_run_291 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState401
      | FINAL ->
          _menhir_run_293 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState401
      | EXISTS ->
          _menhir_run_295 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState401
      | EOF ->
          let _v_0 = _menhir_action_067 () in
          _menhir_run_402 _menhir_stack _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_277 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_OBSERVED (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState277
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState277
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState277
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState277
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState277
      | EOF | SEMI | TOKAND | WITH ->
          let _v = _menhir_action_167 () in
          _menhir_run_278 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState277 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_098 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState098 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NUM _v ->
          _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_099 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          _menhir_run_100 _menhir_stack _menhir_lexbuf _menhir_lexer
      | _ ->
          _eRR ()
  
  and _menhir_run_100 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_NUM -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState100 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NAME _v ->
          _menhir_run_101 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_102 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_048 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EQUAL | RBRK | SEMI ->
          let _1 = _v in
          let _v = _menhir_action_134 _1 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_278 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | TOKAND ->
          let _menhir_stack = MenhirCell1_obsone (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState279
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState279
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState279
          | SYMB_REG _v_0 ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState279
          | PROC _v_1 ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState279
          | NUM _v_2 ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState279
          | NAME _v_3 ->
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState279
          | LBRK ->
              _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState279
          | FAULT ->
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState279
          | EOF | SEMI | TOKAND | WITH ->
              let _v_4 = _menhir_action_167 () in
              _menhir_run_278 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState279 _tok
          | _ ->
              _eRR ())
      | EOF | SEMI | WITH ->
          let _1 = _v in
          let _v = _menhir_action_165 _1 in
          _menhir_goto_obs _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_285 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_NOT (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | EXISTS ->
          let _menhir_s = MenhirState286 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TRUE ->
              _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | SYMB_REG _v ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | PROC _v ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NUM _v ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NOT ->
              _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NAME _v ->
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LPAR ->
              _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | LBRK ->
              _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | FAULT ->
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | FALSE ->
              _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_095 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_174 () in
      _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_prop : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState295 ->
          _menhir_run_296 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState293 ->
          _menhir_run_294 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState291 ->
          _menhir_run_292 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState288 ->
          _menhir_run_289 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState286 ->
          _menhir_run_287 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState094 ->
          _menhir_run_275 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState096 ->
          _menhir_run_274 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState272 ->
          _menhir_run_273 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState200 ->
          _menhir_run_201 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState198 ->
          _menhir_run_199 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState097 ->
          _menhir_run_196 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_296 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_EXISTS as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | SEMI | WITH ->
          let MenhirCell1_EXISTS (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_069 _2 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_198 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_prop -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState198 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_096 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_NOT (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState096 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_097 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState097 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_195 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_175 () in
      _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_200 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_prop -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState200 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_272 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_prop -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState272 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_294 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_FINAL as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | SEMI | WITH ->
          let MenhirCell1_FINAL (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_071 _2 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_292 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_FORALL as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | SEMI | WITH ->
          let MenhirCell1_FORALL (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_068 _2 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_289 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | RPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LPAR (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_072 _2 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer
      | _ ->
          _eRR ()
  
  and _menhir_run_287 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NOT as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | SEMI | WITH ->
          let MenhirCell1_NOT (_menhir_stack, _menhir_s) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_070 _3 in
          _menhir_goto_constr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_275 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_FILTER as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | SEMI | WITH ->
          let MenhirCell1_FILTER (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_084 _2 in
          _menhir_goto_filter _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_filter : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState380 ->
          _menhir_run_382 _menhir_stack _v _tok
      | MenhirState093 ->
          _menhir_run_276 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_276 : type  ttv_stack. ((ttv_stack, _menhir_box_constraints) _menhir_cell1_locations as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_filter (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | OBSERVED ->
          _menhir_run_277 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState276
      | NOT ->
          _menhir_run_285 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState276
      | LPAR ->
          _menhir_run_288 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState276
      | FORALL ->
          _menhir_run_291 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState276
      | FINAL ->
          _menhir_run_293 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState276
      | EXISTS ->
          _menhir_run_295 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState276
      | EOF | SEMI | WITH ->
          let _v_0 = _menhir_action_067 () in
          _menhir_run_311 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState276 _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_288 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState288 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_291 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_FORALL (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState291 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_293 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_FINAL (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState293 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_295 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_EXISTS (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState295 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_274 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NOT -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NOT (_menhir_stack, _menhir_s) = _menhir_stack in
      let _2 = _v in
      let _v = _menhir_action_177 _2 in
      _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_273 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_prop as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
          let MenhirCell1_prop (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_178 _1 _3 in
          _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_201 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_prop as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
          let MenhirCell1_prop (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_180 _1 _3 in
          _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_199 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_prop as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
          let MenhirCell1_prop (_menhir_stack, _menhir_s, _1) = _menhir_stack in
          let _3 = _v in
          let _v = _menhir_action_179 _1 _3 in
          _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_196 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LPAR as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | RPAR ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LPAR (_menhir_stack, _menhir_s) = _menhir_stack in
          let _2 = _v in
          let _v = _menhir_action_181 _2 in
          _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | OR ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_198 _menhir_stack _menhir_lexbuf _menhir_lexer
      | IMPLIES ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_200 _menhir_stack _menhir_lexbuf _menhir_lexer
      | AND ->
          let _menhir_stack = MenhirCell1_prop (_menhir_stack, _menhir_s, _v) in
          _menhir_run_272 _menhir_stack _menhir_lexbuf _menhir_lexer
      | _ ->
          _eRR ()
  
  and _menhir_run_386 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_main_loc_constr) _menhir_state -> _ -> _menhir_box_main_loc_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locations (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | OBSERVED ->
          _menhir_run_277 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState386
      | NOT ->
          _menhir_run_285 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState386
      | LPAR ->
          _menhir_run_288 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState386
      | FORALL ->
          _menhir_run_291 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState386
      | FINAL ->
          _menhir_run_293 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState386
      | EXISTS ->
          _menhir_run_295 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState386
      | EOF ->
          let _v_0 = _menhir_action_067 () in
          _menhir_run_387 _menhir_stack _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_093 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_constraints) _menhir_state -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_locations (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | FILTER ->
          _menhir_run_094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState093
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED | SEMI | WITH ->
          let _v_0 = _menhir_action_083 () in
          _menhir_run_276 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState093 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_094 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_FILTER (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState094 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TRUE ->
          _menhir_run_095 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NOT ->
          _menhir_run_096 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LPAR ->
          _menhir_run_097 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | LBRK ->
          _menhir_run_098 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FAULT ->
          _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | FALSE ->
          _menhir_run_195 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_283 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_atom_prop -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_atom_prop (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_168 _1 _3 in
      _menhir_goto_obsone _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_obsone : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState282 ->
          _menhir_run_283 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState279 ->
          _menhir_run_278 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState277 ->
          _menhir_run_278 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_271 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_176 _1 in
      _menhir_goto_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_258 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_032 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_217 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_035 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_208 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_locindex -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_locindex (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_044 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_206 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_locindex -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_locindex (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_045 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_204 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_locindex -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_locindex (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_046 _1 _3 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_194 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_033 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_173 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_034 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_117 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_036 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_219 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _1 = _v in
      let _v = _menhir_action_122 _1 in
      _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_220 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | SYMB_REG _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _1_inlined1 = _v in
          let _v = _menhir_action_129 _1_inlined1 in
          _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | PROC _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | COLON ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | SYMB_REG _v_0 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let (_3, _1_inlined1) = (_v_0, _v) in
                  let _v = _menhir_action_127 _1_inlined1 _3 in
                  _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PTX_REG_DEC ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_PROC (_menhir_stack, _v) in
                  let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState223) in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | PTX_REG_TYPE _v ->
                      let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                      let _menhir_s = MenhirState226 in
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | NAME _v ->
                          _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                      | DOLLARNAME _v ->
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | NAME _v_4 ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_PROC (_menhir_stack, _v) in
                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState223
              | DOLLARNAME _v_5 ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_PROC (_menhir_stack, _v) in
                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState223
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | NUM _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | COLON ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | SYMB_REG _v_6 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let (_3, _1_inlined1) = (_v_6, _v) in
                  let _v = _menhir_action_128 _1_inlined1 _3 in
                  _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PTX_REG_DEC ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_NUM (_menhir_stack, _v) in
                  let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState230) in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | PTX_REG_TYPE _v ->
                      let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                      let _menhir_s = MenhirState233 in
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | NAME _v ->
                          _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                      | DOLLARNAME _v ->
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | NAME _v_10 ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_NUM (_menhir_stack, _v) in
                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState230
              | DOLLARNAME _v_11 ->
                  let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
                  let _menhir_stack = MenhirCell0_NUM (_menhir_stack, _v) in
                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_11 MenhirState230
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | NAME _v ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let _2 = _v in
          let _v = _menhir_action_132 _2 in
          _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_237 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | SYMB_REG _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _3) = (_v, _v_0) in
              let _v = _menhir_action_120 _1 _3 in
              _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PTX_REG_DEC ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState238) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | PTX_REG_TYPE _v ->
                  let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                  let _menhir_s = MenhirState241 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | DOLLARNAME _v ->
                      _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v_4 ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 MenhirState238
          | DOLLARNAME _v_5 ->
              let _menhir_stack = MenhirCell1_PROC (_menhir_stack, _menhir_s, _v) in
              _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState238
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_244 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | SYMB_REG _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let (_1, _3) = (_v, _v_0) in
              let _v = _menhir_action_121 _1 _3 in
              _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PTX_REG_DEC ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState245) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | PTX_REG_TYPE _v ->
                  let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                  let _menhir_s = MenhirState248 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | DOLLARNAME _v ->
                      _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NUM _v_4 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4
          | NAME _v_5 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_040 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState245
          | DOLLARNAME _v_6 ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState245
          | _ ->
              _eRR ())
      | AND | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
          let _1 = _v in
          let _v = _menhir_action_158 _1 in
          _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_135 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState135 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_OA ->
          _menhir_run_136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_149 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | ATTRS ->
          _menhir_run_156 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_136 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_TOK_OA (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_PTE ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let name = _v in
                          let _v = _menhir_action_173 name in
                          _menhir_goto_output_address _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | TOK_PA ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | NAME _v ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | RPAR ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let name = _v in
                          let _v = _menhir_action_172 name in
                          _menhir_goto_output_address _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | NAME _v ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let name = _v in
              let _v = _menhir_action_171 name in
              _menhir_goto_output_address _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_goto_output_address : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_TOK_OA -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let _menhir_stack = MenhirCell0_output_address (_menhir_stack, _v) in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          _menhir_run_148 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState147
      | RPAR ->
          let _v_0 = _menhir_action_186 () in
          _menhir_run_166 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_148 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_COMMA (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState148 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_OA ->
          _menhir_run_136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NAME _v ->
          _menhir_run_149 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | ATTRS ->
          _menhir_run_156 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_149 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          _menhir_run_148 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState149
      | COLON ->
          let _menhir_stack = MenhirCell1_COLON (_menhir_stack, MenhirState149) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NUM _v_0 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _1 = _v_0 in
              let _v = _menhir_action_164 _1 in
              _menhir_goto_name_or_num _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
          | NAME _v_2 ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _1 = _v_2 in
              let _v = _menhir_action_163 _1 in
              _menhir_goto_name_or_num _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
          | _ ->
              _eRR ())
      | RPAR ->
          let _v_4 = _menhir_action_186 () in
          _menhir_run_155 _menhir_stack _menhir_lexbuf _menhir_lexer _v_4 _tok
      | _ ->
          _eRR ()
  
  and _menhir_goto_name_or_num : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NAME, ttv_result) _menhir_cell1_COLON -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let _menhir_stack = MenhirCell0_name_or_num (_menhir_stack, _v) in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          _menhir_run_148 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState153
      | RPAR ->
          let _v_0 = _menhir_action_186 () in
          _menhir_run_154 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_154 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_NAME, ttv_result) _menhir_cell1_COLON _menhir_cell0_name_or_num -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_name_or_num (_menhir_stack, v) = _menhir_stack in
      let MenhirCell1_COLON (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, key) = _menhir_stack in
      let tail = _v in
      let _v = _menhir_action_183 key tail v in
      _menhir_goto_prop_head _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_prop_head : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState395 ->
          _menhir_run_396 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState135 ->
          _menhir_run_167 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | MenhirState148 ->
          _menhir_run_165 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_396 : type  ttv_stack. (ttv_stack, _menhir_box_pteval) _menhir_cell1_LPAR -> _ -> _ -> _ -> _ -> _menhir_box_pteval =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_LPAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let pteval = _v in
      let _v = _menhir_action_188 pteval in
      _menhir_goto_pteval _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_pteval : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState394 ->
          _menhir_run_398 _menhir_stack _v
      | MenhirState358 ->
          _menhir_run_365 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState335 ->
          _menhir_run_340 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState260 ->
          _menhir_run_266 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState218 ->
          _menhir_run_257 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState174 ->
          _menhir_run_193 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState119 ->
          _menhir_run_172 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_365 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_029 loc v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_340 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_028 loc typ v in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_266 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_037 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_257 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_038 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_193 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_039 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_172 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
      let v = _v in
      let _v = _menhir_action_040 loc v in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_167 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LPAR -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LPAR (_menhir_stack, _menhir_s) = _menhir_stack in
      let pteval = _v in
      let _v = _menhir_action_188 pteval in
      _menhir_goto_pteval _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_165 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_COMMA -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_COMMA (_menhir_stack, _menhir_s) = _menhir_stack in
      let pteval = _v in
      let _v = _menhir_action_187 pteval in
      _menhir_goto_prop_tail _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_prop_tail : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState147 ->
          _menhir_run_166 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState163 ->
          _menhir_run_164 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState149 ->
          _menhir_run_155 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState153 ->
          _menhir_run_154 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_166 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_TOK_OA _menhir_cell0_output_address -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell0_output_address (_menhir_stack, oa) = _menhir_stack in
      let MenhirCell1_TOK_OA (_menhir_stack, _menhir_s) = _menhir_stack in
      let tail = _v in
      let _v = _menhir_action_182 oa tail in
      _menhir_goto_prop_head _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_164 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ATTRS, ttv_result) _menhir_cell1_separated_nonempty_list_COMMA_NAME_ -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_separated_nonempty_list_COMMA_NAME_ (_menhir_stack, _, attrs) = _menhir_stack in
      let MenhirCell1_ATTRS (_menhir_stack, _menhir_s) = _menhir_stack in
      let tail = _v in
      let _v = _menhir_action_185 attrs tail in
      _menhir_goto_prop_head _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_155 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NAME -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, a) = _menhir_stack in
      let tail = _v in
      let _v = _menhir_action_184 a tail in
      _menhir_goto_prop_head _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_156 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_ATTRS (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COLON ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | LPAR ->
              let _menhir_s = MenhirState158 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | NAME _v ->
                  _menhir_run_159 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_159 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState160 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NAME _v ->
              _menhir_run_159 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | RPAR ->
          let x = _v in
          let _v = _menhir_action_199 x in
          _menhir_goto_separated_nonempty_list_COMMA_NAME_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_goto_separated_nonempty_list_COMMA_NAME_ : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      match _menhir_s with
      | MenhirState158 ->
          _menhir_run_162 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | MenhirState160 ->
          _menhir_run_161 _menhir_stack _menhir_lexbuf _menhir_lexer _v
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_162 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_ATTRS as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _menhir_stack = MenhirCell1_separated_nonempty_list_COMMA_NAME_ (_menhir_stack, _menhir_s, _v) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | COMMA ->
          _menhir_run_148 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState163
      | RPAR ->
          let _v_0 = _menhir_action_186 () in
          _menhir_run_164 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_161 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_NAME -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, x) = _menhir_stack in
      let xs = _v in
      let _v = _menhir_action_200 x xs in
      _menhir_goto_separated_nonempty_list_COMMA_NAME_ _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
  
  and _menhir_run_254 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState254 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NUM _v ->
          _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_261 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_092 () in
      _menhir_goto_instr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_instr : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState358 ->
          _menhir_run_368 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState335 ->
          _menhir_run_342 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | MenhirState260 ->
          _menhir_run_269 _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_368 : type  ttv_stack. (ttv_stack, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let i = _v in
      let _v = _menhir_action_016 i loc in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_342 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_left_loc -> _ -> _ -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_left_loc (_menhir_stack, _, loc) = _menhir_stack in
      let MenhirCell1_NAME (_menhir_stack, _menhir_s, _) = _menhir_stack in
      let i = _v in
      let _v = _menhir_action_017 i loc in
      _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_269 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_location -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      let MenhirCell1_location (_menhir_stack, _menhir_s, loc) = _menhir_stack in
      let i = _v in
      let _v = _menhir_action_030 i loc in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_265 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let i = _v in
      let _v = _menhir_action_093 i in
      _menhir_goto_instr _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_085 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_191 _1 in
      _menhir_goto_rloc _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_315 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_STAR (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState315 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | LBRK ->
          _menhir_run_316 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_316 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, _menhir_s) in
      let _menhir_s = MenhirState316 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | NUM _v ->
          _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  and _menhir_run_326 : type  ttv_stack. ttv_stack -> _ -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState326
      | TOK_PTE ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState326
      | TOK_PA ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState326
      | SYMB_REG _v_0 ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState326
      | STAR ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_stack = MenhirCell1_STAR (_menhir_stack, MenhirState326) in
          let _menhir_s = MenhirState327 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | SYMB_REG _v ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | PROC _v ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NUM _v ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LBRK ->
              _menhir_run_316 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | PROC _v_5 ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v_5 MenhirState326
      | NUM _v_6 ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v_6 MenhirState326
      | NAME _v_7 ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v_7 MenhirState326
      | LBRK ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, MenhirState326) in
          let _menhir_s = MenhirState332 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              let _menhir_stack = MenhirCell1_NUM (_menhir_stack, _menhir_s, _v) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | RBRK ->
                  _menhir_run_047 _menhir_stack _menhir_lexbuf _menhir_lexer
              | COLON ->
                  _menhir_run_100 _menhir_stack _menhir_lexbuf _menhir_lexer
              | _ ->
                  _eRR ())
          | NAME _v ->
              _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | _ ->
              _eRR ())
      | COLON ->
          let _menhir_stack = MenhirCell1_NAME (_menhir_stack, _menhir_s, _v) in
          _menhir_run_048 _menhir_stack _menhir_lexbuf _menhir_lexer
      | EOF | EQUAL | SEMI ->
          let _1 = _v in
          let _v = _menhir_action_134 _1 in
          _menhir_goto_location_global _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_348 : type  ttv_stack. ttv_stack -> _ -> _ -> (ttv_stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _menhir_stack = MenhirCell1_ATOMIC (_menhir_stack, _menhir_s) in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | NAME _v ->
          let _menhir_stack = MenhirCell0_NAME (_menhir_stack, _v) in
          let _menhir_s = MenhirState349 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | SYMB_REG _v ->
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | PROC _v ->
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NUM _v ->
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LBRK ->
              _menhir_run_316 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_263 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_location, ttv_result) _menhir_cell1_LCURLY -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LCURLY (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _4 = _v in
      let _v = _menhir_action_041 _1 _4 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_252 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_location, ttv_result) _menhir_cell1_LCURLY -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LCURLY (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _4 = _v in
      let _v = _menhir_action_042 _1 _4 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_215 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_location, ttv_result) _menhir_cell1_LCURLY -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let MenhirCell1_LCURLY (_menhir_stack, _) = _menhir_stack in
      let MenhirCell1_location (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _4 = _v in
      let _v = _menhir_action_043 _1 _4 in
      _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_214 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_maybev_notag -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v ->
      let MenhirCell1_maybev_notag (_menhir_stack, _menhir_s, _1) = _menhir_stack in
      let _3 = _v in
      let _v = _menhir_action_156 _1 _3 in
      _menhir_goto_maybev_list _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
  
  and _menhir_run_116 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_151 _1 in
      _menhir_goto_maybev _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_350 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_ATOMIC _menhir_cell0_NAME as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState351 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell0_NAME (_menhir_stack, typ) = _menhir_stack in
          let MenhirCell1_ATOMIC (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_015 loc typ in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_334 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_NAME as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
          let _menhir_s = MenhirState335 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | VALUE _v ->
              _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | TOK_TAG ->
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PTE ->
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_PA ->
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | TOK_NOP ->
              _menhir_run_261 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NUM _v ->
              _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | NAME _v ->
              _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | LPAR ->
              _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | INSTR _v ->
              _menhir_run_265 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | COLON ->
              _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | ATOMICINIT ->
              let _menhir_stack = MenhirCell1_ATOMICINIT (_menhir_stack, _menhir_s) in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | LPAR ->
                  let _menhir_s = MenhirState337 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | VALUE _v ->
                      _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | TOK_TAG ->
                      _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PTE ->
                      _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PA ->
                      _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | NUM _v ->
                      _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | NAME _v ->
                      _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | COLON ->
                      _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | _ ->
                      _eRR ())
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_014 loc typ in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_328 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | AMPER ->
              _menhir_run_322 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState329
          | COLON | NAME _ | NUM _ | TOK_PA | TOK_PTE | TOK_TAG | VALUE _ ->
              let _v_0 = _menhir_action_009 () in
              _menhir_run_330 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState329 _tok
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell1_STAR (_menhir_stack, _) = _menhir_stack in
          let MenhirCell1_NAME (_menhir_stack, _menhir_s, typ) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_024 loc typ in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_322 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc as 'stack) -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      let _v = _menhir_action_008 () in
      _menhir_goto_amperopt _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_goto_amperopt : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match _menhir_s with
      | MenhirState329 ->
          _menhir_run_330 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | MenhirState321 ->
          _menhir_run_323 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _menhir_fail ()
  
  and _menhir_run_330 : type  ttv_stack. ((((ttv_stack, _menhir_box_init) _menhir_cell1_NAME, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_amperopt (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | VALUE _v_0 ->
          _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState330
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState330
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState330
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState330
      | NUM _v_1 ->
          _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState330
      | NAME _v_2 ->
          _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState330
      | COLON ->
          _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState330
      | _ ->
          _eRR ()
  
  and _menhir_run_323 : type  ttv_stack. (((ttv_stack, _menhir_box_init) _menhir_cell1_STAR, _menhir_box_init) _menhir_cell1_left_loc as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _menhir_stack = MenhirCell1_amperopt (_menhir_stack, _menhir_s, _v) in
      match (_tok : MenhirBasics.token) with
      | VALUE _v_0 ->
          _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState323
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState323
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState323
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState323
      | NUM _v_1 ->
          _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v_1 MenhirState323
      | NAME _v_2 ->
          _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_2 MenhirState323
      | COLON ->
          _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState323
      | _ ->
          _eRR ()
  
  and _menhir_run_320 : type  ttv_stack. ((ttv_stack, _menhir_box_init) _menhir_cell1_STAR as 'stack) -> _ -> _ -> _ -> ('stack, _menhir_box_init) _menhir_state -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | EQUAL ->
          let _menhir_stack = MenhirCell1_left_loc (_menhir_stack, _menhir_s, _v) in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | AMPER ->
              _menhir_run_322 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState321
          | COLON | NAME _ | NUM _ | TOK_PA | TOK_PTE | TOK_TAG | VALUE _ ->
              let _v_0 = _menhir_action_009 () in
              _menhir_run_323 _menhir_stack _menhir_lexbuf _menhir_lexer _v_0 MenhirState321 _tok
          | _ ->
              _eRR ())
      | EOF | SEMI ->
          let MenhirCell1_STAR (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_026 loc in
          _menhir_goto_atom_init _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_255 : type  ttv_stack ttv_result. (ttv_stack, ttv_result) _menhir_cell1_LBRK -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc = _v in
          let _v = _menhir_action_133 loc in
          _menhir_goto_location_deref _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_191 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_LBRK -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LBRK (_menhir_stack, _) = _menhir_stack in
          let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc_inlined1 = _v in
          let _v = _menhir_action_049 loc loc_inlined1 in
          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_170 : type  ttv_stack ttv_result. (((ttv_stack, ttv_result) _menhir_cell1_LBRK, ttv_result) _menhir_cell1_location_global, ttv_result) _menhir_cell1_LBRK -> _ -> _ -> _ -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          let MenhirCell1_LBRK (_menhir_stack, _) = _menhir_stack in
          let MenhirCell1_location_global (_menhir_stack, _, loc) = _menhir_stack in
          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
          let loc_inlined1 = _v in
          let _v = _menhir_action_050 loc loc_inlined1 in
          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
      | _ ->
          _eRR ()
  
  and _menhir_run_118 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_160 _1 in
      _menhir_goto_maybev_notag _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  and _menhir_run_103 : type  ttv_stack ttv_result. ((ttv_stack, ttv_result) _menhir_cell1_LBRK as 'stack) -> _ -> _ -> _ -> ('stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      match (_tok : MenhirBasics.token) with
      | RBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | NOTEQUAL ->
              let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
              let _menhir_s = MenhirState105 in
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v ->
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | TOK_TAG ->
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PTE ->
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | TOK_PA ->
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | NUM _v ->
                  _menhir_run_107 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | NAME _v ->
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
              | COLON ->
                  _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
              | _ ->
                  _eRR ())
          | EQUALEQUAL ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v_3 ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_3 MenhirState119
              | TOK_TAG ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | TOK_PTE ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | TOK_PA ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | SYMB_REG _v_4 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                  let (loc, _1_inlined2) = (_v, _v_4) in
                  let _v = _menhir_action_062 _1_inlined2 loc in
                  _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PROC _v_5 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | COLON ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | SYMB_REG _v_6 ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                          let (loc, _3_inlined1, _1_inlined2) = (_v, _v_6, _v_5) in
                          let _v = _menhir_action_060 _1_inlined2 _3_inlined1 loc in
                          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | PTX_REG_DEC ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState119, _v_5) in
                          let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState122) in
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | PTX_REG_TYPE _v ->
                              let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                              let _menhir_s = MenhirState125 in
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              (match (_tok : MenhirBasics.token) with
                              | NAME _v ->
                                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | DOLLARNAME _v ->
                                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | _ ->
                                  _eRR ())
                          | _ ->
                              _eRR ())
                      | NAME _v_10 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState119, _v_5) in
                          _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_10 MenhirState122
                      | DOLLARNAME _v_11 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState119, _v_5) in
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_11 MenhirState122
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | NUM _v_12 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | COLON ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | SYMB_REG _v_13 ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                          let (loc, _3_inlined1, _1_inlined2) = (_v, _v_13, _v_12) in
                          let _v = _menhir_action_061 _1_inlined2 _3_inlined1 loc in
                          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | PTX_REG_DEC ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState119, _v_12) in
                          let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState129) in
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | PTX_REG_TYPE _v ->
                              let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                              let _menhir_s = MenhirState132 in
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              (match (_tok : MenhirBasics.token) with
                              | NAME _v ->
                                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | DOLLARNAME _v ->
                                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | _ ->
                                  _eRR ())
                          | _ ->
                              _eRR ())
                      | NUM _v_17 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState119, _v_12) in
                          _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v_17
                      | NAME _v_18 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState119, _v_12) in
                          _menhir_run_040 _menhir_stack _menhir_lexbuf _menhir_lexer _v_18 MenhirState129
                      | DOLLARNAME _v_19 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState119, _v_12) in
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_19 MenhirState129
                      | _ ->
                          _eRR ())
                  | AND | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
                      let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                      let _v_20 =
                        let _1 = _v_12 in
                        _menhir_action_158 _1
                      in
                      _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v_20 MenhirState119 _tok
                  | _ ->
                      _eRR ())
              | NAME _v_21 ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_21 MenhirState119
              | LPAR ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | LBRK ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, MenhirState119) in
                  let _menhir_s = MenhirState169 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | TOK_TAG ->
                      _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PTE ->
                      _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PA ->
                      _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | NUM _v ->
                      _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | NAME _v ->
                      _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | COLON ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState119
              | _ ->
                  _eRR ())
          | EQUAL ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              (match (_tok : MenhirBasics.token) with
              | VALUE _v_24 ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_106 _menhir_stack _menhir_lexbuf _menhir_lexer _v_24 MenhirState174
              | TOK_TAG ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | TOK_PTE ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | TOK_PA ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | SYMB_REG _v_25 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                  let (loc, _1_inlined2) = (_v, _v_25) in
                  let _v = _menhir_action_055 _1_inlined2 loc in
                  _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
              | PROC _v_26 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | COLON ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | SYMB_REG _v_27 ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                          let (loc, _3_inlined1, _1_inlined2) = (_v, _v_27, _v_26) in
                          let _v = _menhir_action_053 _1_inlined2 _3_inlined1 loc in
                          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | PTX_REG_DEC ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState174, _v_26) in
                          let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState177) in
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | PTX_REG_TYPE _v ->
                              let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                              let _menhir_s = MenhirState180 in
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              (match (_tok : MenhirBasics.token) with
                              | NAME _v ->
                                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | DOLLARNAME _v ->
                                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | _ ->
                                  _eRR ())
                          | _ ->
                              _eRR ())
                      | NAME _v_31 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState174, _v_26) in
                          _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v_31 MenhirState177
                      | DOLLARNAME _v_32 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_PROC (_menhir_stack, MenhirState174, _v_26) in
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_32 MenhirState177
                      | _ ->
                          _eRR ())
                  | _ ->
                      _eRR ())
              | NUM _v_33 ->
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | COLON ->
                      let _tok = _menhir_lexer _menhir_lexbuf in
                      (match (_tok : MenhirBasics.token) with
                      | SYMB_REG _v_34 ->
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          let MenhirCell1_LBRK (_menhir_stack, _menhir_s) = _menhir_stack in
                          let (loc, _3_inlined1, _1_inlined2) = (_v, _v_34, _v_33) in
                          let _v = _menhir_action_054 _1_inlined2 _3_inlined1 loc in
                          _menhir_goto_atom_prop _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
                      | PTX_REG_DEC ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState174, _v_33) in
                          let _menhir_stack = MenhirCell1_PTX_REG_DEC (_menhir_stack, MenhirState184) in
                          let _tok = _menhir_lexer _menhir_lexbuf in
                          (match (_tok : MenhirBasics.token) with
                          | PTX_REG_TYPE _v ->
                              let _menhir_stack = MenhirCell0_PTX_REG_TYPE (_menhir_stack, _v) in
                              let _menhir_s = MenhirState187 in
                              let _tok = _menhir_lexer _menhir_lexbuf in
                              (match (_tok : MenhirBasics.token) with
                              | NAME _v ->
                                  _menhir_run_030 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | DOLLARNAME _v ->
                                  _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                              | _ ->
                                  _eRR ())
                          | _ ->
                              _eRR ())
                      | NUM _v_38 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState174, _v_33) in
                          _menhir_run_109 _menhir_stack _menhir_lexbuf _menhir_lexer _v_38
                      | NAME _v_39 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState174, _v_33) in
                          _menhir_run_040 _menhir_stack _menhir_lexbuf _menhir_lexer _v_39 MenhirState184
                      | DOLLARNAME _v_40 ->
                          let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                          let _menhir_stack = MenhirCell1_NUM (_menhir_stack, MenhirState174, _v_33) in
                          _menhir_run_031 _menhir_stack _menhir_lexbuf _menhir_lexer _v_40 MenhirState184
                      | _ ->
                          _eRR ())
                  | AND | EOF | EXISTS | FINAL | FORALL | IMPLIES | LPAR | NOT | OBSERVED | OR | RPAR | SEMI | WITH ->
                      let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                      let _v_41 =
                        let _1 = _v_33 in
                        _menhir_action_158 _1
                      in
                      _menhir_run_116 _menhir_stack _menhir_lexbuf _menhir_lexer _v_41 MenhirState174 _tok
                  | _ ->
                      _eRR ())
              | NAME _v_42 ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_110 _menhir_stack _menhir_lexbuf _menhir_lexer _v_42 MenhirState174
              | LPAR ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_135 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | LBRK ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  let _menhir_stack = MenhirCell1_LBRK (_menhir_stack, MenhirState174) in
                  let _menhir_s = MenhirState190 in
                  let _tok = _menhir_lexer _menhir_lexbuf in
                  (match (_tok : MenhirBasics.token) with
                  | TOK_TAG ->
                      _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PTE ->
                      _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | TOK_PA ->
                      _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
                  | NUM _v ->
                      _menhir_run_099 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | NAME _v ->
                      _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
                  | _ ->
                      _eRR ())
              | COLON ->
                  let _menhir_stack = MenhirCell1_location_global (_menhir_stack, _menhir_s, _v) in
                  _menhir_run_114 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState174
              | _ ->
                  _eRR ())
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  and _menhir_run_084 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> _ -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok ->
      let _1 = _v in
      let _v = _menhir_action_117 _1 in
      _menhir_goto_location _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
  
  let _menhir_run_001 : type  ttv_stack ttv_result. ttv_stack -> _ -> _ -> (ttv_stack, ttv_result) _menhir_state -> ttv_result =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LBRK ->
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_TAG ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState002
          | TOK_PTE ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState002
          | TOK_PA ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState002
          | SYMB_REG _v ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState002
          | RBRK ->
              let _tok = _menhir_lexer _menhir_lexbuf in
              let _v = _menhir_action_144 () in
              _menhir_goto_locations _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s _tok
          | PROC _v ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState002
          | NUM _v ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState002
          | NAME _v ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_044 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState002
          | FAULT ->
              let _menhir_stack = MenhirCell1_LOCATIONS (_menhir_stack, _menhir_s) in
              _menhir_run_051 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState002
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  let _menhir_run_000 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_constraints =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LOCATIONS ->
          _menhir_run_001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState000
      | EOF | EXISTS | FILTER | FINAL | FORALL | LPAR | NOT | OBSERVED | SEMI | WITH ->
          let _v = _menhir_action_145 () in
          _menhir_run_093 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState000 _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_314 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_init =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState314
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState314
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState314
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState314
      | STAR ->
          _menhir_run_315 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState314
      | SEMI ->
          _menhir_run_325 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState314
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState314
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState314
      | NAME _v ->
          _menhir_run_326 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState314
      | LBRK ->
          _menhir_run_316 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState314
      | ATOMIC ->
          _menhir_run_348 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState314
      | EOF ->
          let _v = _menhir_action_088 () in
          _menhir_run_369 _menhir_stack _v _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_376 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | OBSERVED ->
          _menhir_run_277 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState376
      | NOT ->
          _menhir_run_285 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState376
      | LPAR ->
          _menhir_run_288 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState376
      | FORALL ->
          _menhir_run_291 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState376
      | FINAL ->
          _menhir_run_293 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState376
      | EXISTS ->
          _menhir_run_295 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState376
      | EOF ->
          let _v = _menhir_action_067 () in
          _menhir_run_378 _menhir_stack _v _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_380 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_filter =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | FILTER ->
          _menhir_run_094 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState380
      | EOF ->
          let _v = _menhir_action_083 () in
          _menhir_run_382 _menhir_stack _v _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_384 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_loc_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LOCATIONS ->
          _menhir_run_001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState384
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED ->
          let _v = _menhir_action_145 () in
          _menhir_run_386 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState384 _tok
      | _ ->
          _eRR ()
  
  let _menhir_run_390 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_main_location =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState390 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | TOK_TAG ->
          _menhir_run_003 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PTE ->
          _menhir_run_010 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | TOK_PA ->
          _menhir_run_019 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
      | SYMB_REG _v ->
          _menhir_run_023 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | PROC _v ->
          _menhir_run_025 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NUM _v ->
          _menhir_run_034 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | NAME _v ->
          _menhir_run_102 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
      | _ ->
          _eRR ()
  
  let _menhir_run_394 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_pteval =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _menhir_s = MenhirState394 in
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LPAR ->
          let _menhir_stack = MenhirCell1_LPAR (_menhir_stack, _menhir_s) in
          let _menhir_s = MenhirState395 in
          let _tok = _menhir_lexer _menhir_lexbuf in
          (match (_tok : MenhirBasics.token) with
          | TOK_OA ->
              _menhir_run_136 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | NAME _v ->
              _menhir_run_149 _menhir_stack _menhir_lexbuf _menhir_lexer _v _menhir_s
          | ATTRS ->
              _menhir_run_156 _menhir_stack _menhir_lexbuf _menhir_lexer _menhir_s
          | _ ->
              _eRR ())
      | _ ->
          _eRR ()
  
  let _menhir_run_399 : type  ttv_stack. ttv_stack -> _ -> _ -> _menhir_box_skip_loc_constr =
    fun _menhir_stack _menhir_lexbuf _menhir_lexer ->
      let _tok = _menhir_lexer _menhir_lexbuf in
      match (_tok : MenhirBasics.token) with
      | LOCATIONS ->
          _menhir_run_001 _menhir_stack _menhir_lexbuf _menhir_lexer MenhirState399
      | EOF | EXISTS | FINAL | FORALL | LPAR | NOT | OBSERVED ->
          let _v = _menhir_action_145 () in
          _menhir_run_401 _menhir_stack _menhir_lexbuf _menhir_lexer _v MenhirState399 _tok
      | _ ->
          _eRR ()
  
end

let skip_loc_constr =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_skip_loc_constr v = _menhir_run_399 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let pteval =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_pteval v = _menhir_run_394 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let main_location =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_main_location v = _menhir_run_390 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let main_loc_constr =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_main_loc_constr v = _menhir_run_384 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let main_filter =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_main_filter v = _menhir_run_380 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let main_constr =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_main_constr v = _menhir_run_376 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let init =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_init v = _menhir_run_314 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v

let constraints =
  fun _menhir_lexer _menhir_lexbuf ->
    let _menhir_stack = () in
    let MenhirBox_constraints v = _menhir_run_000 _menhir_stack _menhir_lexbuf _menhir_lexer in
    v
